Imports System.ComponentModel
Imports System.Data.SqlClient

#Region "DatabaseLicenseProvider"
'\\ --[DatabaseLicenseProvider]-------------------------------------------------
'\\ Implements System.ComponentModel.LicenseProvider for licenses
'\\ held in a database
'\\ The instance being licensed can have an attribute <LicenseDatabaseSource()>
'\\ which tells the database license providor which table to look in and 
'\\ which fields to match on...
'\\ ----------------------------------------------------------------------------
Public Class DatabaseLicenseProvider
    Inherits LicenseProvider

#Region "GetLicense"
    Public Overrides Function GetLicense(ByVal context As System.ComponentModel.LicenseContext, ByVal type As System.Type, ByVal instance As Object, ByVal allowExceptions As Boolean) As System.ComponentModel.License

    End Function
#End Region

End Class

#End Region

#Region "PUMALicenseValidator"
<DatabaseLicenseSourceTableAttribute("Print Servers"), _
 DatabaseLicenseLookupFieldAttribute("Name"), _
 DatabaseLicenseLicenseFieldAttribute("License"), _
 DatabaseLicensePasswordFieldAttribute("Password")> _
Public Class PUMALicenseValidator
    Inherits License

#Region "Private members"
    Private _ServerName As String
    Private _LicenseKey As Guid
    Private _Password As String
#End Region

#Region "Public interface"
    Public ReadOnly Property ServerName() As String
        Get
            Return _ServerName
        End Get
    End Property


    Public ReadOnly Property Valid() As Boolean
        Get
            '\\ Returns true if the password is valid for this license key
            Dim GUIDBytes() As Byte = _LicenseKey.ToByteArray

            If _Password.Length = 16 Then
                For nIndex As Integer = 0 To 15
                    If IsPasswordCharInvalid(GUIDBytes(nIndex), _Password.Chars(15 - nIndex)) Then
                        Return False
                        Exit Property
                    End If
                    Return True
                Next
            Else
                '\\ Only 16-char passwords can be valid at all
                Return False
            End If

        End Get
    End Property
#End Region

#Region "Public constructors"
    Public Sub New(ByVal ServerName As String, _
                   ByVal LicenseKey As Guid, _
                   ByVal Password As String)

        _ServerName = ServerName
        _LicenseKey = LicenseKey
        _Password = Password

    End Sub

    Public Sub New(ByVal ServerName As String)

        _ServerName = ServerName
        '\\ Read the license key and password from the database table
        Dim TableName As String
        Dim LookupField As String
        Dim LicenseField As String
        Dim PasswordField As String

        Try
            Dim DatabaseTable As DatabaseLicenseSourceTableAttribute = Attribute.GetCustomAttribute(Me.GetType, GetType(DatabaseLicenseSourceTableAttribute))
            TableName = DatabaseTable.TableName
            Dim LookupFieldAttr As DatabaseLicenseLookupFieldAttribute = Attribute.GetCustomAttribute(Me.GetType, GetType(DatabaseLicenseLookupFieldAttribute))
            LookupField = LookupFieldAttr.Fieldname
            Dim LicenseFieldAttr As DatabaseLicenseLicenseFieldAttribute = Attribute.GetCustomAttribute(Me.GetType, GetType(DatabaseLicenseLicenseFieldAttribute))
            LicenseField = LicenseFieldAttr.Fieldname
            Dim PasswordFieldAttr As DatabaseLicensePasswordFieldAttribute = Attribute.GetCustomAttribute(Me.GetType, GetType(DatabaseLicensePasswordFieldAttribute))
            PasswordField = PasswordFieldAttr.Fieldname
        Catch ex As Exception
            TableName = "Print Servers"
            LookupField = "Name"
            LicenseField = "License"
            PasswordField = "Password"
        End Try

        '\\ Get the data from the fields in the table [Databasetable]
        Dim SSQL As String = "SELECT " & MakeValidName(LicenseField) & "," & MakeValidName(PasswordField) & _
                             " FROM " & MakeValidName(TableName) & _
                             " WHERE " & MakeValidName(LookupField) & " = '" & ServerName & "'"

        Dim _dbconn As New SqlConnection(My.Settings.ConnectionString)
        If _dbconn.State <> ConnectionState.Open Then
            _dbconn.Open()
        End If

        Dim sqlCmd As New SqlCommand(SSQL, _dbconn)
        Dim sqlRd As SqlDataReader = sqlCmd.ExecuteReader
        If sqlRd.Read Then
            _LicenseKey = sqlRd.GetGuid(0)
            _Password = sqlRd.GetString(1)
        End If

        _dbconn.Close()
    End Sub
#End Region

#Region "Private properties"
    Private Function IsPasswordCharInvalid(ByVal btIn As Byte, ByVal chIn As Char) As Boolean
        If btIn Mod 2 = 0 Then
            If Not Char.IsLetter(chIn) Then
                Return True
            End If
        Else
            If Not Char.IsDigit(chIn) Then
                Return True
            End If
        End If
    End Function
#End Region

#Region "License overrides"
    Public Overrides Sub Dispose()

    End Sub

    Public Overrides ReadOnly Property LicenseKey() As String
        Get
            If Me.Valid Then
                Return _LicenseKey.ToString
            Else
                Return "Unlicensed server"
            End If
        End Get
    End Property
#End Region

#Region "MakeValidName"
    Public Shared Function MakeValidName(ByVal Name As String) As String

        If Name.IndexOf(" ") > -1 Then
            If Name.IndexOf("[") = -1 Then
                Return "[" & Name & "]"
            Else
                Return Name
            End If
        Else
            Return Name
        End If

    End Function
#End Region

End Class
#End Region

#Region "DatabaseLicenseSourceTableAttribute"
<AttributeUsage(AttributeTargets.Class, AllowMultiple:=False)> _
Public Class DatabaseLicenseSourceTableAttribute
    Inherits Attribute

#Region "Private properties"
    Private _TableName As String
#End Region

#Region "Public interface"
    Public ReadOnly Property TableName() As String
        Get
            Return _TableName
        End Get
    End Property
#End Region

#Region "Public constructor"
    Public Sub New(ByVal TableName As String)
        _TableName = TableName
    End Sub
#End Region

End Class
#End Region

#Region "DatabaseLicenseLookupFieldAttribute"
<AttributeUsage(AttributeTargets.Class, AllowMultiple:=False)> _
Public Class DatabaseLicenseLookupFieldAttribute
    Inherits Attribute

#Region "Private properties"
    Private _Fieldname As String
#End Region

#Region "Public interface"
    Public ReadOnly Property Fieldname() As String
        Get
            Return _Fieldname
        End Get
    End Property
#End Region

#Region "Public constructor"
    Public Sub New(ByVal Fieldname As String)
        _Fieldname = Fieldname
    End Sub
#End Region

End Class
#End Region

#Region "DatabaseLicenseLicenseFieldAttribute"
<AttributeUsage(AttributeTargets.Class, AllowMultiple:=False)> _
Public Class DatabaseLicenseLicenseFieldAttribute
    Inherits Attribute

#Region "Private properties"
    Private _Fieldname As String
#End Region

#Region "Public interface"
    Public ReadOnly Property Fieldname() As String
        Get
            Return _Fieldname
        End Get
    End Property
#End Region

#Region "Public constructor"
    Public Sub New(ByVal Fieldname As String)
        _Fieldname = Fieldname
    End Sub
#End Region

End Class
#End Region

#Region "DatabaseLicensePasswordFieldAttribute"
<AttributeUsage(AttributeTargets.Class, AllowMultiple:=False)> _
Public Class DatabaseLicensePasswordFieldAttribute
    Inherits Attribute

#Region "Private properties"
    Private _Fieldname As String
#End Region

#Region "Public interface"
    Public ReadOnly Property Fieldname() As String
        Get
            Return _Fieldname
        End Get
    End Property
#End Region

#Region "Public constructor"
    Public Sub New(ByVal Fieldname As String)
        _Fieldname = Fieldname
    End Sub
#End Region

End Class
#End Region