Imports System.Data.SqlClient

#Region "DatabaseView"
'\\ --[DatabaseView]-------------------------------------------------
'\\
'\\ -----------------------------------------------------------------
Public Class DatabaseView

#Region "Private constants"
    Const SQL_GETFIELDS As String = "select distinct Column_name = isnull(c.name, 'NULL'), " & _
                          "  Type = isnull(t.name, '') ," & _
                          "  Length = isnull(c.length, 0) , " & _
                          "  Prec = isnull(c.prec, 0) , " & _
                          "  Scale = isnull(c.scale, 0) ," & _
                          "  Nulls = convert(bit, (isnull(c.status,0) & 8)) " & _
                          "  from syscolumns c, systypes t " & _
                          "  where c.id = object_id('%1%') and c.xtype = t.xtype AND T.NAME != 'sysname' "

#End Region

#Region "Private properties"
    Private _Name As String
    Private _Fields As DatabaseFields
#End Region

#Region "Public interface"

#Region "Name"
    Public ReadOnly Property Name() As String
        Get
            Return _Name
        End Get
    End Property
#End Region

#Region "Fields"
    Public ReadOnly Property Fields() As DatabaseFields
        Get
            If _Fields Is Nothing Then
                _Fields = New DatabaseFields(SQL_GETFIELDS.Replace("%1%", _Name))
            End If
            Return _Fields
        End Get
    End Property
#End Region

#Region "ToString"
    Public Overrides Function ToString() As String
        Return _Name
    End Function
#End Region

#End Region

#Region "Public constructors"
    Public Sub New(ByVal Viewname As String)
        _Name = StripName(Viewname)
    End Sub
#End Region

#Region "Private methods"
    Private Function StripName(ByVal Viewname As String) As String

        If Viewname.StartsWith("dbo.") Then
            Viewname = Viewname.Substring(5)
        End If
        Viewname = Viewname.Replace("[", "")
        Viewname = Viewname.Replace("]", "")

        Return Viewname.Trim
    End Function

#End Region

End Class
#End Region

#Region "DatabaseViews"
Public Class DatabaseViews
    Inherits CollectionBase

#Region "Private constants"
    Const SQL_GETVIEWS As String = "select name from sysobjects where type = 'V' and category = 0 "
#End Region

#Region "Public constructors"
    Public Sub New()

        Dim dbConn As New SqlConnection(My.Settings.ConnectionString)

        If dbConn.State <> ConnectionState.Open Then
            dbConn.Open()
        End If

        Try
            '\\ Execute the SQL_GETVIEWS
            '\\ and add each result to the List collection
            Dim sqlCmd As New SqlCommand(SQL_GETVIEWS, dbConn)
            Dim sqlRdr As SqlDataReader = sqlCmd.ExecuteReader
            While sqlRdr.Read
                Me.List.Add(New DatabaseView(sqlRdr.GetString(0)))
            End While
        Catch ex As Exception
            If Puma_Manager_Application.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, Me.GetType.ToString)
            End If
        Finally
            dbConn.Close()
        End Try

    End Sub
#End Region

End Class
#End Region

#Region "DatabaseFields"
Public Class DatabaseFields
    Inherits DictionaryBase

#Region "Public interface"
    Public Function ContainsField(ByVal Fieldname As String) As Boolean
        Return Me.Dictionary.Contains(Fieldname)
    End Function

    Public Function GetItem(ByVal Fieldname As String) As DatabaseField
        Return Me.Dictionary.Item(Fieldname)
    End Function

    Public Function Values() As ICollection
        Return Me.Dictionary.Values
    End Function
#End Region

#Region "Public constructor"
    Public Sub New(ByVal QueryDef As String)

        Dim dbConn As New SqlConnection(My.Settings.ConnectionString)
        If Not dbConn.State = ConnectionState.Open Then
            dbConn.Open()
        End If

        Try
            Dim dbcmd As New SqlCommand(QueryDef, dbConn)
            Dim dbRdr As SqlDataReader = dbcmd.ExecuteReader
            While dbRdr.Read
                '\\ Add the field to the internal list
                Me.Dictionary.Add(dbRdr.GetString(0), New DatabaseField(dbRdr.GetString(0), dbRdr.GetString(1), dbRdr.GetBoolean(5)))
            End While
        Catch ex As Exception
            If Puma_Manager_Application.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, Me.GetType.ToString)
            End If
        Finally
            dbConn.Close()
        End Try
    End Sub
#End Region
End Class
#End Region

#Region "DatabaseField"
Public Class DatabaseField

#Region "Private members"
    Private _Name As String
    Private _Type As String
    Private _Nulls As Boolean
    Private _Selected As Boolean
#End Region

#Region "Public interface"

#Region "Name"
    Public ReadOnly Property Name() As String
        Get
            Return _Name
        End Get
    End Property
#End Region

#Region "Type"
    Public ReadOnly Property Type() As String
        Get
            Return _Type
        End Get
    End Property
#End Region

#Region "Nulls"
    Public ReadOnly Property Nulls() As Boolean
        Get
            Return _Nulls
        End Get
    End Property
#End Region

#Region "Selected"
    Public Property Selected() As Boolean
        Get
            Return _Selected
        End Get
        Set(ByVal Value As Boolean)
            _Selected = Value
        End Set
    End Property
#End Region

#Region "ToString"
    Public Overrides Function ToString() As String
        Return _Name & "(" & _Type & ")"
    End Function
#End Region

#End Region

#Region "Public constructors"
    Public Sub New(ByVal Name As String, ByVal Type As String, ByVal Nulls As Boolean)
        _Name = Name
        _Type = Type
        _Nulls = Nulls
        _Selected = True
    End Sub
#End Region

End Class
#End Region