Imports System.Data.SqlClient
Imports System.Text

#Region "PrinterUsageReportDefinition"
'\\ --[PrinterUsageReportDefinition]--------------------------------------------
'\\ Class to represent a single entry in  the table [Printer Usage Reports] 
'\\ 
'\\ ----------------------------------------------------------------------------
Public Class PrinterUsageReportDefinition

#Region "Private enumerated types"
    Private Enum PrinterUsageReportsFieldPositions
        Report_Name = 0
        Query_Definition = 1
        Description = 2
        RestrictByPrinter = 3
        RestrictByDocumentName = 4
        RestrictByTimeRange = 5
        RestrictByUser = 6
        Type = 7
        Parameters = 8
        CrystalReportsFilename = 9
    End Enum
#End Region

#Region "Private properties"
    Private _ReportName As String
    Private _QueryDefinition As String
    Private _Description As String

    Private _WhereClause As String

    Private _RecordSource As DatabaseView
#End Region

#Region "Public interface"

#Region "Name"
    Public ReadOnly Property Name() As String
        Get
            Return _ReportName
        End Get
    End Property
#End Region

#Region "Description"
    Public Property Description() As String
        Get
            Return _Description
        End Get
        Set(ByVal Value As String)
            _Description = Value
        End Set
    End Property
#End Region

#Region "QueryDefinition"
    Public ReadOnly Property QueryDefinition() As String
        Get
            Dim sRet As New StringBuilder
            Dim sFields As New StringBuilder
            Dim bOneNotSelected As Boolean

            sRet.Append("SELECT ")
            For Each fld As DatabaseField In Me.RecordSource.Fields.Values
                If fld.Selected Then
                    sFields.Append(SQLUtilities.MakeValidName(fld.Name))
                    sFields.Append(" , ")
                Else
                    bOneNotSelected = True
                End If
            Next

            If Not bOneNotSelected Then
                sRet.Append(" * ")
            Else
                sRet.Append(sFields.ToString.Substring(0, sFields.Length - 3))
            End If

            sRet.Append(" FROM ")
            sRet.Append(SQLUtilities.MakeValidName(RecordSource.Name))

            If Me.WhereClause <> "" Then
                sRet.Append(" WHERE ")
                sRet.Append(WhereClause)
            End If

            Return sRet.ToString
        End Get
    End Property
#End Region

#Region "RecordSource"
    Public Property RecordSource() As DatabaseView
        Get
            If _RecordSource Is Nothing Then
                _RecordSource = GetDataSourceFromQueryDef(_QueryDefinition)
                Call SetFieldListFromQueryDef(_QueryDefinition)
            End If
            Return _RecordSource
        End Get
        Set(ByVal Value As DatabaseView)
            _RecordSource = Value
        End Set
    End Property
#End Region

#Region "WhereClause"
    Public Property WhereClause() As String
        Get
            If _WhereClause = "" Then
                _WhereClause = GetWhereClause(_QueryDefinition)
            End If
            Return _WhereClause
        End Get
        Set(ByVal Value As String)
            _WhereClause = Value
        End Set
    End Property
#End Region

#Region "CanRestrictByPrinter"
    Public ReadOnly Property CanRestrictByPrinter() As Boolean
        Get
            If Me.RecordSource.Fields.ContainsField("Printer Name") Then
                If Me.RecordSource.Fields.GetItem("Printer Name").Selected Then
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        End Get
    End Property
#End Region

#Region "CanRestrictByUser"
    Public ReadOnly Property CanRestrictByUser() As Boolean
        Get
            If Me.RecordSource.Fields.ContainsField("User Name") Then
                If Me.RecordSource.Fields.GetItem("User Name").Selected Then
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        End Get
    End Property
#End Region

#Region "CanRestrictByDocumentName"
    Public ReadOnly Property CanRestrictByDocumentName() As Boolean
        Get
            If Me.RecordSource.Fields.ContainsField("Documen Name") Then
                If Me.RecordSource.Fields.GetItem("Document Name").Selected Then
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        End Get
    End Property
#End Region

#Region "CanRestrictByTimeRange"
    Public ReadOnly Property CanRestrictByDateRange() As Boolean
        Get
            If Me.RecordSource.Fields.ContainsField("Submitted") Then
                If Me.RecordSource.Fields.GetItem("Submitted").Selected Then
                    Return True
                Else
                    Return False
                End If
            Else
                Return False
            End If
        End Get
    End Property
#End Region

#Region "SaveToDatabase"
    Public Sub SaveToDatabase()

        Dim dbConn As New SqlConnection(My.Settings.ConnectionString)

        If dbConn.State <> ConnectionState.Open Then
            dbConn.Open()
        End If

        Dim Exists As Integer
        Try
            Dim cmdExists As New SqlCommand("SELECT COUNT(*) FROM dbo.[Printer Usage Reports] WHERE [Report Name] = '" & _ReportName & "'", dbConn)
            Exists = cmdExists.ExecuteScalar
        Catch ex As Exception
            If Puma_Manager_Application.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, Me.GetType.ToString)
            End If
            dbConn.Close()
            Exit Sub
        End Try


        Dim ssql As String
        If Exists > 0 Then
            '\\ Update the existing report
            ssql = "UPDATE dbo.[printer Usage Reports] " & _
                   " SET [Query Definition] = " & SQLUtilities.QuoteString(Me.QueryDefinition) & _
                   " , Description = " & SQLUtilities.QuoteString(Me.Description) & _
                   " , [Restrict By Printer] = " & IIf(Me.CanRestrictByPrinter, "1", "0") & _
                   " , [Restrict By Document Name] = " & IIf(Me.CanRestrictByDocumentName, "1", "0") & _
                   " , [Restrict By Time Range] = " & IIf(Me.CanRestrictByDateRange, "1", "0") & _
                   " , [Restrict By User] = " & IIf(Me.CanRestrictByUser, "1", "0") & _
                   " WHERE [Report Name] = '" & _ReportName & "'"
        Else
            '\\ Insert a new report
            ssql = "INSERT INTO dbo.[printer Usage Reports] " & _
                   "([Report Name], [Query Definition], Description, " & _
                   " [Restrict By Printer], [Restrict By Document Name], [Restrict By Time Range], [Restrict By User]) " & _
                   " VALUES (" & _
                   SQLUtilities.QuoteString(Me.Name) & "," & _
                   SQLUtilities.QuoteString(Me.QueryDefinition) & "," & _
                   SQLUtilities.QuoteString(Me.Description) & "," & _
                   IIf(Me.CanRestrictByPrinter, "1", "0") & "," & _
                   IIf(Me.CanRestrictByDocumentName, "1", "0") & "," & _
                   IIf(Me.CanRestrictByDateRange, "1", "0") & "," & _
                   IIf(Me.CanRestrictByUser, "1", "0") & ")"
        End If

        Try
            Dim dbCmd As New SqlCommand(ssql, dbConn)
            If dbCmd.ExecuteNonQuery() > 0 Then
                If Puma_Manager_Application.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine(ssql & " succeeded ")
                End If
            Else
                If Puma_Manager_Application.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine(ssql & " updated zero rows ")
                End If
            End If

        Catch ex As Exception
            If Puma_Manager_Application.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, Me.GetType.ToString)
            End If
        Finally
            dbConn.Close()
        End Try

    End Sub
#End Region

#End Region

#Region "Shared Interface"
    Public Shared Function CreateNewReport(ByVal Reportname As String) As PrinterUsageReportDefinition

        Dim dbconn As New SqlConnection(My.Settings.ConnectionString)

        If dbconn.State <> ConnectionState.Open Then
            dbconn.Open()
        End If

        Try

        Catch ex As Exception

        Finally
            dbconn.Close()
        End Try

        Return New PrinterUsageReportDefinition(Reportname)

    End Function

    Public Shared Sub DeleteReport(ByVal ReportName As String)

        Dim dbconn As New SqlConnection(My.Settings.ConnectionString)

        If dbconn.State <> ConnectionState.Open Then
            dbconn.Open()
        End If

        Try
            Dim ssql As String = "DELETE dbo.[Printer Usage Reports] WHERE [Report Name] = " & SQLUtilities.QuoteString(ReportName)
            Dim dbCmd As New SqlCommand(ssql, dbconn)
            If dbCmd.ExecuteNonQuery > 0 Then
                If Puma_Manager_Application.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine(ssql & " deleted rows", "DeleteReport")
                End If
            Else
                If Puma_Manager_Application.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine(ssql & " deleted zero rows", "DeleteReport")
                End If
            End If
        Catch ex As Exception
            If Puma_Manager_Application.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, "DeleteReport")
            End If
        Finally
            dbconn.Close()
        End Try

    End Sub
#End Region

#Region "Public constructor"
    Public Sub New(ByVal ReportName As String)
        _ReportName = ReportName
        '\\ Read the other properties from the table [Printer Usage Reports]
        Call RefreshFromDB()
    End Sub
#End Region

#Region "Private methods"
    Private Sub RefreshFromDB()

        Dim dbConn As New SqlConnection(My.Settings.ConnectionString)

        If dbConn.State <> ConnectionState.Open Then
            dbConn.Open()
        End If

        Try
            Dim sqlCmd As New SqlCommand("SELECT * FROM [Printer Usage Reports] WHERE [Report Name] = '" & _ReportName & "'", dbConn)
            Dim dbRdr As SqlDataReader = sqlCmd.ExecuteReader
            If dbRdr.Read Then
                _Description = dbRdr.GetString(PrinterUsageReportsFieldPositions.Description)
                _QueryDefinition = dbRdr.GetString(PrinterUsageReportsFieldPositions.Query_Definition)
                _WhereClause = dbRdr.GetString(PrinterUsageReportsFieldPositions.Parameters)
            End If
        Catch ex As Exception
            If Puma_Manager_Application.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, Me.GetType.ToString)
            End If
        Finally
            dbConn.Close()
        End Try

    End Sub

    Private Function GetDataSourceFromQueryDef(ByVal QueryDef As String) As DatabaseView

        Dim StartIndex As Integer = QueryDef.IndexOf(" FROM ") + 5
        Dim EndIndex As Integer
        Dim Viewname As String

        If QueryDef.IndexOf(" WHERE ") > 0 Then
            EndIndex = QueryDef.IndexOf(" WHERE ")
        ElseIf QueryDef.IndexOf(" ORDER BY ") > 0 Then
            EndIndex = QueryDef.IndexOf(" ORDER BY ")
        Else
            EndIndex = QueryDef.Length
        End If

        Viewname = QueryDef.Substring(StartIndex, (EndIndex - StartIndex)).Trim
        Return New DatabaseView(Viewname)

    End Function

    Private Sub SetFieldListFromQueryDef(ByVal QueryDef As String)

        '\\ Takes the "SELECT ... FROM" part of the query and sets the 
        '\\ Fields.Selected properties appropriately
        Dim QueryPart As String

        If QueryDef.StartsWith("SELECT") Then
            QueryPart = QueryDef.Substring(7)
        Else
            Throw New ArgumentException("Incomplete query definition", Me.GetType.ToString)
            Exit Sub
        End If

        If QueryPart.IndexOf(" FROM ") > 0 Then
            QueryPart = QueryPart.Substring(0, QueryPart.IndexOf(" FROM "))
        End If

        For Each fld As DatabaseField In _RecordSource.Fields.Values
            If QueryPart.Trim = "*" Then
                fld.Selected = True
            Else
                If QueryPart.IndexOf(fld.Name) = -1 Then
                    fld.Selected = False
                Else
                    fld.Selected = True
                End If
            End If
        Next fld

    End Sub

    Private Function GetWhereClause(ByVal QueryDef As String) As String

        If QueryDef.IndexOf(" WHERE ") > 0 Then
            If QueryDef.IndexOf(" GROUP BY ") > 0 OrElse QueryDef.IndexOf(" ORDER BY ") > 0 Then
                Dim nEnd As Integer = QueryDef.IndexOf(" GROUP BY ")
                If nEnd < 0 Then
                    nEnd = QueryDef.IndexOf(" ORDER BY ")
                End If
                Return QueryDef.Substring(QueryDef.IndexOf(" WHERE ") + 6, nEnd - (QueryDef.IndexOf(" WHERE ") + 6))
            Else
                Return QueryDef.Substring(QueryDef.IndexOf(" WHERE ") + 6)
            End If
        Else
            Return ""
        End If
    End Function
#End Region

End Class
#End Region
