Imports System.Data.SqlClient

'\\ --[PrinterUsageReports]-------------------------------
'\\ Individual printer usage report
'\\ ------------------------------------------------------
Public Class PrinterUsageReport

#Region "Private members"
    Private _Name As String
    Private _Description As String
    Private _QueryDef As String
    Private _CanRestrictByPrinter As Boolean
    Private _CanRestrictByUser As Boolean
    Private _CanRestrictByDocumentName As Boolean
    Private _CanRestrictByDateRange As Boolean
#End Region

#Region "Public interface"
    Public ReadOnly Property Name() As String
        Get
            Return _Name
        End Get
    End Property

    Public ReadOnly Property Description() As String
        Get
            Return _Description
        End Get
    End Property

    Public ReadOnly Property QueryDef() As String
        Get
            Return _QueryDef
        End Get
    End Property

    Public ReadOnly Property CanRestrictByPrinter() As Boolean
        Get
            Return _CanRestrictByPrinter
        End Get
    End Property

    Public ReadOnly Property CanRestrictByUser() As Boolean
        Get
            Return _CanRestrictByUser
        End Get
    End Property

    Public ReadOnly Property CanRestrictByDocumentName() As Boolean
        Get
            Return _CanRestrictByDocumentName
        End Get
    End Property

    Public ReadOnly Property CanRestrictByDateRange() As Boolean
        Get
            Return _CanRestrictByDateRange
        End Get
    End Property

    Public Overrides Function ToString() As String
        Return Name
    End Function
#End Region

#Region "Constructors"
    Public Sub New(ByVal Name As String, _
                   ByVal QueryDef As String, _
                   ByVal Description As String, _
                   ByVal CanRestrictByPrinter As Boolean, _
                   ByVal CanRestrictByUser As Boolean, _
                   ByVal CanRestrictByDocumentName As Boolean, _
                   ByVal CanRestrictByDateRange As Boolean)

        If Puma_Manager_Application.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("New (" & Name & "," & _
                            QueryDef & ")", Me.GetType.ToString)
        End If

        _Name = Name
        _QueryDef = QueryDef
        _Description = Description
        _CanRestrictByPrinter = CanRestrictByPrinter
        _CanRestrictByUser = CanRestrictByUser
        _CanRestrictByDocumentName = CanRestrictByDocumentName
        _CanRestrictByDateRange = CanRestrictByDateRange


    End Sub
#End Region

End Class

'\\ --[PrinterUsageReportsDictionary]---------------------
'\\ Collection of printer usage reports keyed by name
'\\ ------------------------------------------------------
Public Class PrinterUsageReportsDictionary
    Inherits DictionaryBase

    Default Public Property Item(ByVal key As [String]) As PrinterUsageReport
        Get
            Return CType(Dictionary(key), PrinterUsageReport)
        End Get
        Set(ByVal Value As PrinterUsageReport)
            Dictionary(key) = Value
        End Set
    End Property

    Public ReadOnly Property Keys() As ICollection
        Get
            Return Dictionary.Keys
        End Get
    End Property

    Public ReadOnly Property Values() As ICollection
        Get
            Return Dictionary.Values
        End Get
    End Property

    Public Overloads Sub Add(ByVal key As [String], ByVal value As PrinterUsageReport)
        Dictionary.Add(key, value)
    End Sub

    Public Overloads Sub Add(ByVal value As PrinterUsageReport)
        dictionary.Add(value.Name, value)
    End Sub

    Public Function Contains(ByVal key As [String]) As Boolean
        Return Dictionary.Contains(key)
    End Function

    Public Sub Remove(ByVal key As [String])
        Dictionary.Remove(key)
    End Sub


#Region "Constructors"
    Public Sub New(ByRef PrinterUsageReports As SqlDataReader)
        With PrinterUsageReports
            While .Read
                Me.Add(New PrinterUsageReport(.GetString(0), .GetString(1), .GetString(2), .GetBoolean(3), .GetBoolean(4), .GetBoolean(5), .GetBoolean(6)))
            End While
        End With
    End Sub

    Public Sub New()

        Dim _dbConn As New SqlConnection(My.Settings.ConnectionString)

        If _dbConn.State <> ConnectionState.Open Then
            _dbConn.Open()
        End If

        Dim sqlCmd As New SqlCommand("SELECT * FROM dbo.[Printer Usage Reports]", _dbConn)
        Dim PrinterUsageReports As SqlDataReader = sqlCmd.ExecuteReader
        With PrinterUsageReports
            While .Read
                Me.Add(New PrinterUsageReport(.GetString(0), .GetString(1), .GetString(2), .GetBoolean(3), .GetBoolean(4), .GetBoolean(5), .GetBoolean(6)))
            End While
        End With

    End Sub
#End Region

End Class

