Imports System.ComponentModel
Imports System.Configuration.Install
Imports System.Reflection
Imports System.IO
Imports System.DirectoryServices


<RunInstaller(True)> Public Class ConfigFileConnectionStringInstaller
    Inherits System.Configuration.Install.Installer

#Region "Private properties"
    Private _DatabaseName As String '\\ Passed as a parameter to the install program...
    Private _ConfigFilename As String
    Private _Servername As String
#End Region

#Region "Private constants"
    Private Const SERVER_STRING As String = "server=(local)"
    Private Const TEST_CONNECT_STRING As String = " Persist Security Info=False;Integrated Security=SSPI;database=master;server=(local);Connect Timeout=30"
#End Region

#Region " Component Designer generated code "

    Public Sub New()
        MyBase.New()

        'This call is required by the Component Designer.
        InitializeComponent()

        'Add any initialization after the InitializeComponent() call

    End Sub

    'Installer overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Component Designer
    'It can be modified using the Component Designer.  
    'Do not modify it using the code editor.
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        components = New System.ComponentModel.Container
    End Sub

#End Region

#Region "Public overrides"
    Public Overrides Sub Install(ByVal statesaver As System.Collections.IDictionary)
        MyBase.Install(statesaver)

        '\\ Get the name of the database to look for
        Try
            If Me.Context.Parameters.Count > 0 Then
                _DatabaseName = Context.Parameters("DatabaseName")
            End If
        Catch ex As Exception
            Try
                Me.Context.LogMessage(ex.ToString)
            Catch exLog As Exception
                Console.WriteLine(ex.ToString)
            End Try
            _DatabaseName = "PrintMonitor"
        End Try

        '\\ Get the name of the .config file that needs to be edited...
        Try
            If Me.Context.Parameters.Count > 0 Then
                _ConfigFilename = Context.Parameters("ConfigFilename")
            End If
            Dim file As New FileInfo(_ConfigFilename)
            If Not file.Exists Then
                Throw New FileNotFoundException("File missing : " & _ConfigFilename)
            End If
        Catch ex As Exception
            Dim apppath As String
            apppath = [Assembly].GetExecutingAssembly.Location
            Dim dir As New System.IO.DirectoryInfo(System.IO.Path.GetDirectoryName(apppath))

            Dim configfiles() As System.IO.FileInfo = dir.GetFiles("*.config")
            If configfiles.Length > 0 Then
                _ConfigFilename = configfiles(0).Name
            Else
                Context.LogMessage("No config file found to change")
                Throw New InstallException("No configuration file found in the installation directory")
            End If

        End Try

        '\\ Find the server that has the named database on it...
        Dim fFeedback As New Form_FindDatabaseServer
        '\\ Show feedback to the user as this may take a while...
        fFeedback.Show()
        '\\ First try local machine

        Dim Servername As String = Environment.MachineName
        fFeedback.Label_Message_Detail.Text = "Checking " & Servername
        fFeedback.Refresh()
        If ServerContainsDatabase(Servername, _DatabaseName) Then
            _Servername = Servername
        Else
            '\\ Check each server on the network...
            Dim de As New DirectorySearcher
            Try
                For Each cde As DirectoryEntry In de.FindAll
                    fFeedback.Label_Message_Detail.Text = "Checking " & cde.Name
                    fFeedback.Refresh()
                    If ServerContainsDatabase(cde.Name, _DatabaseName) Then
                        _Servername = cde.Name
                        Exit For
                    End If
                Next
            Catch ex As Exception
                '\\ Could not find the server...

            End Try
        End If

        If _Servername = "" Then
            Try
                Me.Context.LogMessage("The database server could not be located for " & _DatabaseName)
            Catch exLog As Exception
                Console.WriteLine("The database server could not be located for " & _DatabaseName)
            End Try
        Else
            Dim configFile As New FileInfo(_ConfigFilename)

            fFeedback.Label_Message_Heading.Text = "Locating config file"
            fFeedback.Label_Message_Detail.Text = _ConfigFilename
            fFeedback.Refresh()

            Try
                Dim reader As New StreamReader(configFile.FullName)
                Dim configFileNew As New FileInfo(_ConfigFilename & ".new")
                Dim strmOut As New FileStream(configFileNew.FullName, FileMode.CreateNew)
                Dim writer As New StreamWriter(strmOut, System.Text.Encoding.UTF8)

                Try
                    Dim sLine As String
                    sLine = reader.ReadLine
                    While Not (sLine Is Nothing)
                        '\\ If the current node is not the ConnectionString write it out as is
                        If sLine.IndexOf(SERVER_STRING) >= 0 Then
                            sLine = ReplaceServerName(sLine, _Servername)
                        End If
                        writer.WriteLine(sLine)
                        sLine = reader.ReadLine
                    End While
                    reader.Close()
                    writer.Close()

                    fFeedback.Label_Message_Heading.Text = "Backing up old config file"
                    fFeedback.Label_Message_Detail.Text = _ConfigFilename
                    fFeedback.Refresh()

                    '\\ Rename the two files
                    File.Move(_ConfigFilename, _ConfigFilename & ".old")
                    File.Move(_ConfigFilename & ".new", _ConfigFilename)

                Catch ex As Exception
                    Try
                        Context.LogMessage(ex.ToString)
                    Catch exlog As Exception
                        Console.WriteLine(ex.ToString)
                    End Try
                    Throw ex
                End Try

            Catch ex As Exception

            End Try
        End If

        fFeedback.Hide()

    End Sub
#End Region

#Region "Private methods"
    Private Function ReplaceServerName(ByVal Line As String, ByVal ServerName As String) As String

        'e.g. Persist Security Info=False;Integrated Security=SSPI;database=PrintMonitor;server=(local);Connect Timeout=30
        If Line.IndexOf(SERVER_STRING) >= 0 Then
            Dim Pos As Integer = Line.IndexOf(SERVER_STRING)
            Return Line.Substring(0, Pos) & "server=" & ServerName & Line.Substring(Pos + SERVER_STRING.Length)
        Else
            Return Line
        End If

    End Function

    Private Function ServerContainsDatabase(ByVal Servername As String, ByVal DatabaseName As String) As String

        Dim _dbconn As New SqlClient.SqlConnection(ReplaceServerName(TEST_CONNECT_STRING, Servername))

        Try
            If _dbconn.State <> ConnectionState.Open Then
                _dbconn.Open()
            End If
            _dbconn.ChangeDatabase(DatabaseName)
            Return True
        Catch ex As System.Data.SqlClient.SqlException
            Return False
        Catch ex As Exception
            Context.LogMessage(ex.ToString)
            Throw
        Finally
            If _dbconn.State = ConnectionState.Open Then
                _dbconn.Close()
            End If
        End Try

    End Function

#End Region

End Class
