#Region "DataViewRestriction"
'\\ --[DataViewRestriction]-------------------------------
'\\ Restricts an existing DataView 
'\\ ------------------------------------------------------
Friend Class DataViewRestriction

#Region "Private members"
    Private _Fieldname As String
    Private _mColItems As New Collection()
#End Region

#Region "Public interface"
    '\\ --[Clear]---------------------------------------
    '\\ Removes all items from the restrict by list
    '\\ ------------------------------------------------
    Public Sub Clear()
        _mColItems = New Collection()
    End Sub

    '\\ --[AddItem]-------------------------------------
    '\\ Adds a restriction item to the list
    '\\ ------------------------------------------------
    Public Sub AddItem(ByVal Value As String)
        _mColItems.Add(Value, Value)
    End Sub

    '\\ --[RemoveItem]-------------------------------------
    '\\ Removes a restriction item to the list
    '\\ ------------------------------------------------
    Public Sub RemoveItem(ByVal Value As String)
        _mColItems.Remove(Value)
    End Sub

    '\\ --[Restriction]------------------------------------
    '\\ 
    '\\ ---------------------------------------------------
    Public ReadOnly Property Restriction() As String
        Get
            Dim sRet As String
            If _mColItems.Count > 0 Then
                sRet = _Fieldname
            End If
            If _mColItems.Count = 1 Then
                sRet = sRet & " = '" & _mColItems.Item(1) & "'"
            Else
                sRet = sRet & " IN ("
                Dim nItem As Integer
                For nItem = 1 To _mColItems.Count
                    sRet = sRet & "'" & _mColItems.Item(nItem) & "'"
                    If nItem < _mColItems.Count Then
                        sRet = sRet & " , "
                    End If
                Next
                sRet = sRet & ")"
            End If
            Return sRet
        End Get
    End Property

    Public ReadOnly Property Unrestricted() As Boolean
        Get
            Return (_mColItems.Count = 0)
        End Get
    End Property

#End Region

#Region "Public constructor"
    Public Sub New(ByVal Fieldname As String)
        _Fieldname = Fieldname
    End Sub
#End Region

End Class
#End Region

#Region "DataViewDateRestriction"
Public Class DataViewDateRestriction

#Region "Private properties"
    Private _Fieldname As String
    Private _StartDate As Date
    Private _EndDate As Date
    Private _UnrestrictedStart As Boolean
    Private _UnrestrictedEnd As Boolean
#End Region

#Region "Start Date"
    Public Property StartDate() As Date
        Get
            Return _StartDate
        End Get
        Set(ByVal Value As Date)
            _StartDate = Value
            _UnrestrictedStart = False
        End Set
    End Property
#End Region

#Region "End Date"
    Public Property EndDate() As Date
        Get
            Return _EndDate
        End Get
        Set(ByVal Value As Date)
            _EndDate = Value
            _UnrestrictedEnd = False
        End Set
    End Property
#End Region

#Region "Unrestericted Start"
    Public Property UnrestrictedStart() As Boolean
        Get
            Return _UnrestrictedStart
        End Get
        Set(ByVal Value As Boolean)
            _UnrestrictedStart = Value
        End Set
    End Property
#End Region

#Region "Unrestricted end"
    Public Property UnrestrictedEnd() As Boolean
        Get
            Return _UnrestrictedEnd
        End Get
        Set(ByVal Value As Boolean)
            _UnrestrictedEnd = Value
        End Set
    End Property
#End Region

#Region "Restriction"
    Public ReadOnly Property Restriction() As String
        Get
            Dim sRet As String
            If Not (_UnrestrictedStart And _UnrestrictedEnd) Then
                sRet = _Fieldname
                If (Not _UnrestrictedStart) And (Not _UnrestrictedEnd) Then
                    sRet = sRet & " >= " & DbDate(_StartDate) & " AND " & _Fieldname & " <=  " & DbDate(_EndDate)
                ElseIf Not _UnrestrictedStart Then
                    sRet = sRet & " >= " & DbDate(_StartDate)
                ElseIf Not _UnrestrictedEnd Then
                    sRet = sRet & " <= " & DbDate(_EndDate)
                End If
            End If
            Return sRet
        End Get
    End Property
#End Region

    Private Function DbDate(ByVal dateIn As Date) As String
        Return "'" & Format$(dateIn, "yyyy-MM-dd") & "'"
    End Function

#Region "Public constructor"
    Public Sub New(ByVal Fieldname As String)
        _Fieldname = Fieldname
        _UnrestrictedStart = True
        _UnrestrictedEnd = True
    End Sub
#End Region
End Class
#End Region

#Region "DataViewDocumentRestriction"
Public Class DataViewDocumentRestriction

#Region "Public enumerator"
    Public Enum RestrictionModifier
        None = 0
        StartsWith = 1
        EndsWith = 2
        Contains = 3
        ExactlyEquals = 4
    End Enum
#End Region

#Region "Private members"
    Private _Fieldname As String
    Private _Modifier As RestrictionModifier
    Private _DocumentName As String
#End Region

#Region "Modifier"
    Public Property Modifier() As RestrictionModifier
        Get
            Return _Modifier
        End Get
        Set(ByVal Value As RestrictionModifier)
            _Modifier = Value
        End Set
    End Property
#End Region

#Region "DocumentName"
    Public Property DocumentName() As String
        Get
            Return _DocumentName
        End Get
        Set(ByVal Value As String)
            _DocumentName = Value
        End Set
    End Property
#End Region

#Region "Restriction"
    Public ReadOnly Property Restriction() As String
        Get
            Select Case _Modifier
                Case RestrictionModifier.None
                    Return ""
                Case RestrictionModifier.ExactlyEquals
                    Return _Fieldname & " = '" & _DocumentName & "'"
                Case RestrictionModifier.StartsWith
                    Return _Fieldname & " LIKE '" & _DocumentName & "%'"
                Case RestrictionModifier.EndsWith
                    Return _Fieldname & " LIKE '%" & _DocumentName & "'"
                Case RestrictionModifier.Contains
                    Return _Fieldname & " LIKE '%" & _DocumentName & "%'"
            End Select
        End Get
    End Property
#End Region

#Region "Public constructor"
    Public Sub New(ByVal Fieldname As String)
        _Fieldname = Fieldname
    End Sub
#End Region

End Class
#End Region