'\\ --[Form_PrinterUsageViewer]---------------------------------------------------
'\\ Main form for performing enquiries against the PrinterMonitoring database
'\\ ------------------------------------------------------------------------------
'\\ (c) 2004 - Merrion Computing Ltd
'\\ ------------------------------------------------------------------------------
Imports System.Data.SqlClient
Imports System.Security.Principal
Imports System.Drawing.Printing
Imports System.ComponentModel
Imports System.ComponentModel.TypeConverter

Public Class Form_PrinterUsageViewer
    Inherits System.Windows.Forms.Form


#Region "Private members"
    Dim dt As New DataTable()
    Private _DBConn As SqlConnection
    Private _Reports As PrinterUsageReportsDictionary
    Private _SelectedReport As PrinterUsageReport
    Private _PrinterRestriction As New DataViewRestriction("[Printer Name]")
    Private _UserRestriction As New DataViewRestriction("[User Name]")
    Private _DateRestriction As New DataViewDateRestriction("Submitted")
    Private _DocumentRestriction As New DataViewDocumentRestriction("[Document Name]")

    '\\ _OnlyOwnDocuments - The user cannot see what other users have printed
    Private _OnlyOwnDocuments As Boolean


    Private _GlobalSettings As PumaSettings

    Private _LoggedInUsername As String

    Private _Printer As DataGridPrinter

#End Region

#Region "Private enumerated types"
    Private Enum ListViewSections
        Printers = 0
        Users = 1
        Documents = 2
        DateRange = 3
        Setup = 4
    End Enum

    '\\ ListViewImages - The indexes of the pictures in Imagelist_Sections
    Private Enum ListViewImages
        Setup = 0
        Documents = 1
        Printers = 2
        Users = 3
        Timerange = 4
        DocumentsFilterd = 5
        PrintersFiltered = 6
        UsersFiltered = 7
        TimerangeFiltered = 8
    End Enum

    Private Enum SaveFileDialogReportFilterIndexes
        SaveAs_XML = 1
        SaveAs_Tab = 2
    End Enum

    Private Enum CellTextHorizontalAlignment
        LeftAlign = 1
        CentreAlign = 2
        RightAlign = 3
    End Enum

    Private Enum CellTextVerticalAlignment
        TopAlign = 1
        MiddleAlign = 2
        BottomAlign = 3
    End Enum

#End Region

#Region "Private Constants"
    '\\ Page component sizes
    Private _FooterHeightPercent = 3
    Private _HeaderHeightPercent = 7
    Private _InterSectionSpacingPercent = 2
    Private _CellGutter As Integer = 5
    Friend WithEvents MenuItem_File_Options As System.Windows.Forms.MenuItem
    Private _PrintFontSize = 9
#End Region

#Region " Windows Form Designer generated code "

    Public Sub New()
        MyBase.New()

        SetStyle(ControlStyles.SupportsTransparentBackColor, True)

        'This call is required by the Windows Form Designer.
        InitializeComponent()

        '\\ Read the different printer usage reports
        Dim sFilename As String

        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Initialising the printer usage reports collection from " & sFilename, "Application")
        End If

        Try
            _DBConn = New SqlConnection(PrinterMonitorViewerMain.ConnectionString)
        Catch edb As SqlClient.SqlException
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(edb.Message, "SQL Connection")
            End If
        Catch eOther As Exception
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(eOther.Message, "Database connection")
            End If
        End Try

        _GlobalSettings = New PumaSettings(PrinterMonitorViewerMain.ConnectionString)

        If _GlobalSettings.SettingExists("Category interval") Then
            Try
                Me.Timer_Automatic_Document_Categorisation.Interval = 1000 * _GlobalSettings.SettingValue("Category interval")
            Catch e As Exception
                '\\ Default to every 60 seconds
                Me.Timer_Automatic_Document_Categorisation.Interval = 60000
                If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                    Trace.WriteLine(e.Message, Me.GetType.ToString)
                End If
            End Try
        Else
            '\\ Disable automatic categroisation of documents
            Me.Timer_Automatic_Document_Categorisation.Enabled = False
        End If

        If _DBConn.State = ConnectionState.Closed Then
            _DBConn.Open()
        End If

        Dim sqlCmd As New SqlCommand("SELECT * FROM [Printer Usage Reports]", _DBConn)
        Dim sqlRpts As SqlDataReader = sqlCmd.ExecuteReader
        _Reports = New PrinterUsageReportsDictionary(sqlRpts)
        sqlRpts.Close()

        Dim rpt As PrinterUsageReport
        For Each rpt In _Reports.Values
            '\\ Add new reports menu item
            ContextMenu_Reports.MenuItems.Add(rpt.Name, AddressOf MenuItem_Report_Selection)
        Next

        '\\ Create the context menus (users, printers)
        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Creating printer context menu")
        End If
        Try
            Call CreatePrintersContextMenu()
        Catch ex As Exception
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.Message, "CreatePrintersContextMenu")
            End If
        End Try

        Try
            Call CreateUsersContextmenu()
        Catch eCM As Exception
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(eCM.Message, "CreateUsersContextmenu")
            End If
        End Try


        '\\ Set the application appearance according to the user authority level
        Dim LoggedInuser As New WindowsPrincipal(WindowsIdentity.GetCurrent())

        _LoggedInUsername = DataGridPrinter.StripDomainFromFullUsername(WindowsIdentity.GetCurrent.Name)

        Try
            If LoggedInuser.Identity.IsAuthenticated Then
                If LoggedInuser.IsInRole("Printer Monitors") OrElse LoggedInuser.IsInRole(WindowsBuiltInRole.Administrator) Then
                    OnlyOwnDocuments = False
                Else
                    OnlyOwnDocuments = True
                End If
            Else
                OnlyOwnDocuments = True
            End If
        Catch exLogin As ArgumentException
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(exLogin.Message, exLogin.Source)
            End If
            OnlyOwnDocuments = True
        End Try

        '\\  -- Customise the form from the application settings
        Dim viewSettings As New System.Configuration.AppSettingsReader
        Try
            DataGrid_Report.PreferredColumnWidth = viewSettings.GetValue("DefaultColumnWidth", Me.DataGrid_Report.PreferredColumnWidth.GetType)
        Catch e As Exception
            '\\ Use default values
            DataGrid_Report.PreferredColumnWidth = 200
        End Try

    End Sub


    'Form overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    'Required by the Windows Form Designer
    Private components As System.ComponentModel.IContainer

    'NOTE: The following procedure is required by the Windows Form Designer
    'It can be modified using the Windows Form Designer.  
    'Do not modify it using the code editor.
    Friend WithEvents ImageList_Sections As System.Windows.Forms.ImageList
    Friend WithEvents MainMenu_PrinterUsageViewer As System.Windows.Forms.MainMenu
    Friend WithEvents MenuItem_File As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Help As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Help_About As System.Windows.Forms.MenuItem
    Friend WithEvents ListView_Sections As System.Windows.Forms.ListView
    Friend WithEvents MenuItem_File_Close As System.Windows.Forms.MenuItem
    Friend WithEvents Panel_Parameters As System.Windows.Forms.Panel
    Friend WithEvents Panel_Data As System.Windows.Forms.Panel
    Friend WithEvents PictureBox_Parameters As System.Windows.Forms.PictureBox
    Friend WithEvents DataGrid_Report As System.Windows.Forms.DataGrid
    Friend WithEvents MenuItem_Report As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Report_Refresh As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Report_Parameters As System.Windows.Forms.MenuItem
    Friend WithEvents ContextMenu_Printers As System.Windows.Forms.ContextMenu
    Friend WithEvents MenuItem_Printers_Filter As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Printers_ShowAll As System.Windows.Forms.MenuItem
    Friend WithEvents ContextMenu_Users As System.Windows.Forms.ContextMenu
    Friend WithEvents MenuItem_Users_Filter As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Users_ShowAll As System.Windows.Forms.MenuItem
    Friend WithEvents Label_SelectedReport As System.Windows.Forms.Label
    Friend WithEvents ContextMenu_Reports As System.Windows.Forms.ContextMenu
    Friend WithEvents Button_SelectReport As System.Windows.Forms.Button
    Friend WithEvents ToolTip_Main As System.Windows.Forms.ToolTip
    Friend WithEvents ContextMenu_Documents As System.Windows.Forms.ContextMenu
    Friend WithEvents ContextMenu_DateRange As System.Windows.Forms.ContextMenu
    Friend WithEvents MenuItem_DocumentName As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_DateReange_All As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_DateRange_Today As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_DateRange_ThisWeek As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_DateRange_ThisMonth As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_DateRange_Custom As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_File_PageSetup As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_File_Print As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem1 As System.Windows.Forms.MenuItem
    Friend WithEvents PageSetupDialogMain As System.Windows.Forms.PageSetupDialog
    Friend WithEvents PrintPreviewDialog_Report As System.Windows.Forms.PrintPreviewDialog
    Friend WithEvents MenuItem_Report_Save As System.Windows.Forms.MenuItem
    Friend WithEvents SaveFileDialog_Report As System.Windows.Forms.SaveFileDialog
    Friend WithEvents ContextMenu_Settings As System.Windows.Forms.ContextMenu
    Friend WithEvents MenuItem_Application_Settings As System.Windows.Forms.MenuItem
    Friend WithEvents MenuItem_Document_Categories As System.Windows.Forms.MenuItem
    Friend WithEvents Timer_Automatic_Document_Categorisation As System.Windows.Forms.Timer
    Friend WithEvents ContextMenu_DataGrid As System.Windows.Forms.ContextMenu
    Friend WithEvents MenuItem_DataGrid_Column_Settings As System.Windows.Forms.MenuItem
    Friend WithEvents HelpProvider_Main As System.Windows.Forms.HelpProvider
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(Form_PrinterUsageViewer))
        Dim ListViewItem1 As System.Windows.Forms.ListViewItem = New System.Windows.Forms.ListViewItem(New String() {"Printers"}, 2, System.Drawing.SystemColors.WindowText, System.Drawing.SystemColors.Window, New System.Drawing.Font("Arial", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte)))
        Dim ListViewItem2 As System.Windows.Forms.ListViewItem = New System.Windows.Forms.ListViewItem(New String() {"Users"}, 3, System.Drawing.SystemColors.WindowText, System.Drawing.SystemColors.Window, New System.Drawing.Font("Arial", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte)))
        Dim ListViewItem3 As System.Windows.Forms.ListViewItem = New System.Windows.Forms.ListViewItem(New String() {"Documents"}, 1, System.Drawing.SystemColors.WindowText, System.Drawing.SystemColors.Window, New System.Drawing.Font("Arial", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte)))
        Dim ListViewItem4 As System.Windows.Forms.ListViewItem = New System.Windows.Forms.ListViewItem(New String() {"Date Range"}, 4, System.Drawing.SystemColors.WindowText, System.Drawing.SystemColors.Window, New System.Drawing.Font("Microsoft Sans Serif", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte)))
        Dim ListViewItem5 As System.Windows.Forms.ListViewItem = New System.Windows.Forms.ListViewItem(New String() {"Settings"}, 0, System.Drawing.SystemColors.WindowText, System.Drawing.SystemColors.Window, New System.Drawing.Font("Arial", 8.25!, System.Drawing.FontStyle.Regular, System.Drawing.GraphicsUnit.Point, CType(0, Byte)))
        Me.ImageList_Sections = New System.Windows.Forms.ImageList(Me.components)
        Me.MainMenu_PrinterUsageViewer = New System.Windows.Forms.MainMenu(Me.components)
        Me.MenuItem_File = New System.Windows.Forms.MenuItem
        Me.MenuItem_File_PageSetup = New System.Windows.Forms.MenuItem
        Me.MenuItem_File_Print = New System.Windows.Forms.MenuItem
        Me.MenuItem1 = New System.Windows.Forms.MenuItem
        Me.MenuItem_File_Close = New System.Windows.Forms.MenuItem
        Me.MenuItem_Report = New System.Windows.Forms.MenuItem
        Me.MenuItem_Report_Refresh = New System.Windows.Forms.MenuItem
        Me.MenuItem_Report_Parameters = New System.Windows.Forms.MenuItem
        Me.MenuItem_Report_Save = New System.Windows.Forms.MenuItem
        Me.MenuItem_Help = New System.Windows.Forms.MenuItem
        Me.MenuItem_Help_About = New System.Windows.Forms.MenuItem
        Me.ListView_Sections = New System.Windows.Forms.ListView
        Me.ContextMenu_Printers = New System.Windows.Forms.ContextMenu
        Me.MenuItem_Printers_Filter = New System.Windows.Forms.MenuItem
        Me.MenuItem_Printers_ShowAll = New System.Windows.Forms.MenuItem
        Me.Panel_Parameters = New System.Windows.Forms.Panel
        Me.PictureBox_Parameters = New System.Windows.Forms.PictureBox
        Me.Panel_Data = New System.Windows.Forms.Panel
        Me.Button_SelectReport = New System.Windows.Forms.Button
        Me.Label_SelectedReport = New System.Windows.Forms.Label
        Me.DataGrid_Report = New System.Windows.Forms.DataGrid
        Me.ContextMenu_DataGrid = New System.Windows.Forms.ContextMenu
        Me.MenuItem_DataGrid_Column_Settings = New System.Windows.Forms.MenuItem
        Me.ContextMenu_Users = New System.Windows.Forms.ContextMenu
        Me.MenuItem_Users_Filter = New System.Windows.Forms.MenuItem
        Me.MenuItem_Users_ShowAll = New System.Windows.Forms.MenuItem
        Me.ContextMenu_Reports = New System.Windows.Forms.ContextMenu
        Me.ToolTip_Main = New System.Windows.Forms.ToolTip(Me.components)
        Me.ContextMenu_Documents = New System.Windows.Forms.ContextMenu
        Me.MenuItem_DocumentName = New System.Windows.Forms.MenuItem
        Me.ContextMenu_DateRange = New System.Windows.Forms.ContextMenu
        Me.MenuItem_DateReange_All = New System.Windows.Forms.MenuItem
        Me.MenuItem_DateRange_Today = New System.Windows.Forms.MenuItem
        Me.MenuItem_DateRange_ThisWeek = New System.Windows.Forms.MenuItem
        Me.MenuItem_DateRange_ThisMonth = New System.Windows.Forms.MenuItem
        Me.MenuItem_DateRange_Custom = New System.Windows.Forms.MenuItem
        Me.PageSetupDialogMain = New System.Windows.Forms.PageSetupDialog
        Me.PrintPreviewDialog_Report = New System.Windows.Forms.PrintPreviewDialog
        Me.SaveFileDialog_Report = New System.Windows.Forms.SaveFileDialog
        Me.ContextMenu_Settings = New System.Windows.Forms.ContextMenu
        Me.MenuItem_Application_Settings = New System.Windows.Forms.MenuItem
        Me.MenuItem_Document_Categories = New System.Windows.Forms.MenuItem
        Me.Timer_Automatic_Document_Categorisation = New System.Windows.Forms.Timer(Me.components)
        Me.HelpProvider_Main = New System.Windows.Forms.HelpProvider
        Me.MenuItem_File_Options = New System.Windows.Forms.MenuItem
        Me.Panel_Parameters.SuspendLayout()
        CType(Me.PictureBox_Parameters, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.Panel_Data.SuspendLayout()
        CType(Me.DataGrid_Report, System.ComponentModel.ISupportInitialize).BeginInit()
        Me.SuspendLayout()
        '
        'ImageList_Sections
        '
        Me.ImageList_Sections.ImageStream = CType(resources.GetObject("ImageList_Sections.ImageStream"), System.Windows.Forms.ImageListStreamer)
        Me.ImageList_Sections.TransparentColor = System.Drawing.Color.Transparent
        Me.ImageList_Sections.Images.SetKeyName(0, "")
        Me.ImageList_Sections.Images.SetKeyName(1, "")
        Me.ImageList_Sections.Images.SetKeyName(2, "")
        Me.ImageList_Sections.Images.SetKeyName(3, "")
        Me.ImageList_Sections.Images.SetKeyName(4, "")
        Me.ImageList_Sections.Images.SetKeyName(5, "")
        Me.ImageList_Sections.Images.SetKeyName(6, "")
        Me.ImageList_Sections.Images.SetKeyName(7, "")
        Me.ImageList_Sections.Images.SetKeyName(8, "")
        '
        'MainMenu_PrinterUsageViewer
        '
        Me.MainMenu_PrinterUsageViewer.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_File, Me.MenuItem_Report, Me.MenuItem_Help})
        '
        'MenuItem_File
        '
        Me.MenuItem_File.Index = 0
        Me.MenuItem_File.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_File_PageSetup, Me.MenuItem_File_Print, Me.MenuItem_File_Options, Me.MenuItem1, Me.MenuItem_File_Close})
        Me.MenuItem_File.Text = "&File"
        '
        'MenuItem_File_PageSetup
        '
        Me.MenuItem_File_PageSetup.Index = 0
        Me.MenuItem_File_PageSetup.Text = "Page Set&up"
        '
        'MenuItem_File_Print
        '
        Me.MenuItem_File_Print.Index = 1
        Me.MenuItem_File_Print.Text = "&Print"
        '
        'MenuItem1
        '
        Me.MenuItem1.Index = 3
        Me.MenuItem1.Text = "-"
        '
        'MenuItem_File_Close
        '
        Me.MenuItem_File_Close.Index = 4
        Me.MenuItem_File_Close.Text = "E&xit"
        '
        'MenuItem_Report
        '
        Me.MenuItem_Report.Index = 1
        Me.MenuItem_Report.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_Report_Refresh, Me.MenuItem_Report_Parameters, Me.MenuItem_Report_Save})
        Me.MenuItem_Report.Text = "&Report"
        '
        'MenuItem_Report_Refresh
        '
        Me.MenuItem_Report_Refresh.Enabled = False
        Me.MenuItem_Report_Refresh.Index = 0
        Me.MenuItem_Report_Refresh.Shortcut = System.Windows.Forms.Shortcut.F5
        Me.MenuItem_Report_Refresh.Text = "Re&fresh"
        '
        'MenuItem_Report_Parameters
        '
        Me.MenuItem_Report_Parameters.Enabled = False
        Me.MenuItem_Report_Parameters.Index = 1
        Me.MenuItem_Report_Parameters.Text = "&Parameters"
        '
        'MenuItem_Report_Save
        '
        Me.MenuItem_Report_Save.Index = 2
        Me.MenuItem_Report_Save.Text = "&Save"
        '
        'MenuItem_Help
        '
        Me.MenuItem_Help.Index = 2
        Me.MenuItem_Help.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_Help_About})
        Me.MenuItem_Help.Text = "&Help"
        '
        'MenuItem_Help_About
        '
        Me.MenuItem_Help_About.Index = 0
        Me.MenuItem_Help_About.Text = "&About"
        '
        'ListView_Sections
        '
        Me.ListView_Sections.BorderStyle = System.Windows.Forms.BorderStyle.FixedSingle
        Me.ListView_Sections.ContextMenu = Me.ContextMenu_Printers
        Me.ListView_Sections.Dock = System.Windows.Forms.DockStyle.Left
        Me.HelpProvider_Main.SetHelpKeyword(Me.ListView_Sections, "Printer")
        Me.HelpProvider_Main.SetHelpNavigator(Me.ListView_Sections, System.Windows.Forms.HelpNavigator.KeywordIndex)
        Me.ListView_Sections.Items.AddRange(New System.Windows.Forms.ListViewItem() {ListViewItem1, ListViewItem2, ListViewItem3, ListViewItem4, ListViewItem5})
        Me.ListView_Sections.LargeImageList = Me.ImageList_Sections
        Me.ListView_Sections.Location = New System.Drawing.Point(0, 0)
        Me.ListView_Sections.MultiSelect = False
        Me.ListView_Sections.Name = "ListView_Sections"
        Me.HelpProvider_Main.SetShowHelp(Me.ListView_Sections, True)
        Me.ListView_Sections.Size = New System.Drawing.Size(96, 441)
        Me.ListView_Sections.TabIndex = 1
        Me.ToolTip_Main.SetToolTip(Me.ListView_Sections, "Filter the currently selected printer usage report")
        Me.ListView_Sections.UseCompatibleStateImageBehavior = False
        '
        'ContextMenu_Printers
        '
        Me.ContextMenu_Printers.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_Printers_Filter, Me.MenuItem_Printers_ShowAll})
        '
        'MenuItem_Printers_Filter
        '
        Me.MenuItem_Printers_Filter.Index = 0
        Me.MenuItem_Printers_Filter.Text = "Filter"
        '
        'MenuItem_Printers_ShowAll
        '
        Me.MenuItem_Printers_ShowAll.Index = 1
        Me.MenuItem_Printers_ShowAll.Text = "Show &All"
        '
        'Panel_Parameters
        '
        Me.Panel_Parameters.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.Panel_Parameters.BackColor = System.Drawing.Color.White
        Me.Panel_Parameters.Controls.Add(Me.PictureBox_Parameters)
        Me.Panel_Parameters.Location = New System.Drawing.Point(96, 56)
        Me.Panel_Parameters.Name = "Panel_Parameters"
        Me.Panel_Parameters.Size = New System.Drawing.Size(576, 96)
        Me.Panel_Parameters.TabIndex = 5
        Me.Panel_Parameters.Visible = False
        '
        'PictureBox_Parameters
        '
        Me.PictureBox_Parameters.BackColor = System.Drawing.Color.LightSalmon
        Me.PictureBox_Parameters.Dock = System.Windows.Forms.DockStyle.Top
        Me.PictureBox_Parameters.Location = New System.Drawing.Point(0, 0)
        Me.PictureBox_Parameters.Name = "PictureBox_Parameters"
        Me.PictureBox_Parameters.Size = New System.Drawing.Size(576, 24)
        Me.PictureBox_Parameters.TabIndex = 1
        Me.PictureBox_Parameters.TabStop = False
        '
        'Panel_Data
        '
        Me.Panel_Data.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.Panel_Data.BackColor = System.Drawing.Color.White
        Me.Panel_Data.Controls.Add(Me.Button_SelectReport)
        Me.Panel_Data.Controls.Add(Me.Label_SelectedReport)
        Me.Panel_Data.Controls.Add(Me.DataGrid_Report)
        Me.Panel_Data.Location = New System.Drawing.Point(0, 0)
        Me.Panel_Data.Name = "Panel_Data"
        Me.Panel_Data.Size = New System.Drawing.Size(672, 441)
        Me.Panel_Data.TabIndex = 6
        '
        'Button_SelectReport
        '
        Me.Button_SelectReport.Anchor = CType((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.Button_SelectReport.BackColor = System.Drawing.Color.LightSalmon
        Me.Button_SelectReport.FlatStyle = System.Windows.Forms.FlatStyle.Flat
        Me.Button_SelectReport.ForeColor = System.Drawing.Color.Salmon
        Me.HelpProvider_Main.SetHelpString(Me.Button_SelectReport, "Select a printer usage report")
        Me.Button_SelectReport.Image = CType(resources.GetObject("Button_SelectReport.Image"), System.Drawing.Image)
        Me.Button_SelectReport.ImageAlign = System.Drawing.ContentAlignment.MiddleRight
        Me.Button_SelectReport.Location = New System.Drawing.Point(624, 0)
        Me.Button_SelectReport.Name = "Button_SelectReport"
        Me.HelpProvider_Main.SetShowHelp(Me.Button_SelectReport, True)
        Me.Button_SelectReport.Size = New System.Drawing.Size(48, 24)
        Me.Button_SelectReport.TabIndex = 0
        Me.ToolTip_Main.SetToolTip(Me.Button_SelectReport, "Select from the printer usage reports")
        Me.Button_SelectReport.UseVisualStyleBackColor = False
        '
        'Label_SelectedReport
        '
        Me.Label_SelectedReport.Anchor = CType(((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.Label_SelectedReport.BackColor = System.Drawing.Color.LightSalmon
        Me.Label_SelectedReport.CausesValidation = False
        Me.HelpProvider_Main.SetHelpString(Me.Label_SelectedReport, "Select a printer usage report")
        Me.Label_SelectedReport.ImageAlign = System.Drawing.ContentAlignment.MiddleRight
        Me.Label_SelectedReport.Location = New System.Drawing.Point(96, 0)
        Me.Label_SelectedReport.Name = "Label_SelectedReport"
        Me.HelpProvider_Main.SetShowHelp(Me.Label_SelectedReport, True)
        Me.Label_SelectedReport.Size = New System.Drawing.Size(576, 24)
        Me.Label_SelectedReport.TabIndex = 0
        Me.Label_SelectedReport.Text = "Select report ..."
        Me.Label_SelectedReport.TextAlign = System.Drawing.ContentAlignment.MiddleLeft
        Me.ToolTip_Main.SetToolTip(Me.Label_SelectedReport, "The currently selected printer usage report")
        '
        'DataGrid_Report
        '
        Me.DataGrid_Report.AlternatingBackColor = System.Drawing.Color.Silver
        Me.DataGrid_Report.Anchor = CType((((System.Windows.Forms.AnchorStyles.Top Or System.Windows.Forms.AnchorStyles.Bottom) _
                    Or System.Windows.Forms.AnchorStyles.Left) _
                    Or System.Windows.Forms.AnchorStyles.Right), System.Windows.Forms.AnchorStyles)
        Me.DataGrid_Report.BackColor = System.Drawing.Color.Gainsboro
        Me.DataGrid_Report.BorderStyle = System.Windows.Forms.BorderStyle.None
        Me.DataGrid_Report.ContextMenu = Me.ContextMenu_DataGrid
        Me.DataGrid_Report.DataMember = ""
        Me.DataGrid_Report.HeaderForeColor = System.Drawing.SystemColors.ControlText
        Me.DataGrid_Report.Location = New System.Drawing.Point(96, 24)
        Me.DataGrid_Report.Name = "DataGrid_Report"
        Me.DataGrid_Report.PreferredColumnWidth = 200
        Me.DataGrid_Report.ReadOnly = True
        Me.DataGrid_Report.Size = New System.Drawing.Size(576, 417)
        Me.DataGrid_Report.TabIndex = 2
        '
        'ContextMenu_DataGrid
        '
        Me.ContextMenu_DataGrid.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_DataGrid_Column_Settings})
        '
        'MenuItem_DataGrid_Column_Settings
        '
        Me.MenuItem_DataGrid_Column_Settings.Index = 0
        Me.MenuItem_DataGrid_Column_Settings.Text = "&Column settings"
        '
        'ContextMenu_Users
        '
        Me.ContextMenu_Users.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_Users_Filter, Me.MenuItem_Users_ShowAll})
        '
        'MenuItem_Users_Filter
        '
        Me.MenuItem_Users_Filter.Index = 0
        Me.MenuItem_Users_Filter.Text = "Filter"
        '
        'MenuItem_Users_ShowAll
        '
        Me.MenuItem_Users_ShowAll.Index = 1
        Me.MenuItem_Users_ShowAll.Text = "Show &All"
        '
        'ContextMenu_Documents
        '
        Me.ContextMenu_Documents.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_DocumentName})
        '
        'MenuItem_DocumentName
        '
        Me.MenuItem_DocumentName.Index = 0
        Me.MenuItem_DocumentName.Text = "&Document name like ...."
        '
        'ContextMenu_DateRange
        '
        Me.ContextMenu_DateRange.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_DateReange_All, Me.MenuItem_DateRange_Today, Me.MenuItem_DateRange_ThisWeek, Me.MenuItem_DateRange_ThisMonth, Me.MenuItem_DateRange_Custom})
        '
        'MenuItem_DateReange_All
        '
        Me.MenuItem_DateReange_All.Checked = True
        Me.MenuItem_DateReange_All.Index = 0
        Me.MenuItem_DateReange_All.RadioCheck = True
        Me.MenuItem_DateReange_All.Text = "&All"
        '
        'MenuItem_DateRange_Today
        '
        Me.MenuItem_DateRange_Today.Index = 1
        Me.MenuItem_DateRange_Today.RadioCheck = True
        Me.MenuItem_DateRange_Today.Text = "&Today"
        '
        'MenuItem_DateRange_ThisWeek
        '
        Me.MenuItem_DateRange_ThisWeek.Index = 2
        Me.MenuItem_DateRange_ThisWeek.RadioCheck = True
        Me.MenuItem_DateRange_ThisWeek.Text = "This &week"
        '
        'MenuItem_DateRange_ThisMonth
        '
        Me.MenuItem_DateRange_ThisMonth.Index = 3
        Me.MenuItem_DateRange_ThisMonth.RadioCheck = True
        Me.MenuItem_DateRange_ThisMonth.Text = "This &month"
        '
        'MenuItem_DateRange_Custom
        '
        Me.MenuItem_DateRange_Custom.Index = 4
        Me.MenuItem_DateRange_Custom.RadioCheck = True
        Me.MenuItem_DateRange_Custom.Text = "&Custom..."
        '
        'PrintPreviewDialog_Report
        '
        Me.PrintPreviewDialog_Report.AutoScrollMargin = New System.Drawing.Size(0, 0)
        Me.PrintPreviewDialog_Report.AutoScrollMinSize = New System.Drawing.Size(0, 0)
        Me.PrintPreviewDialog_Report.ClientSize = New System.Drawing.Size(400, 300)
        Me.PrintPreviewDialog_Report.Enabled = True
        Me.PrintPreviewDialog_Report.Icon = CType(resources.GetObject("PrintPreviewDialog_Report.Icon"), System.Drawing.Icon)
        Me.PrintPreviewDialog_Report.Name = "PrintPreviewDialog_Report"
        Me.PrintPreviewDialog_Report.Visible = False
        '
        'SaveFileDialog_Report
        '
        Me.SaveFileDialog_Report.Filter = "XML Files|*.xml|Tab Seperated Values|*.tab"
        Me.SaveFileDialog_Report.OverwritePrompt = False
        Me.SaveFileDialog_Report.Title = "Save Printer Usage Report"
        '
        'ContextMenu_Settings
        '
        Me.ContextMenu_Settings.MenuItems.AddRange(New System.Windows.Forms.MenuItem() {Me.MenuItem_Application_Settings, Me.MenuItem_Document_Categories})
        '
        'MenuItem_Application_Settings
        '
        Me.MenuItem_Application_Settings.Index = 0
        Me.MenuItem_Application_Settings.Text = "&Application Settings"
        '
        'MenuItem_Document_Categories
        '
        Me.MenuItem_Document_Categories.Index = 1
        Me.MenuItem_Document_Categories.Text = "&Document Categories "
        '
        'Timer_Automatic_Document_Categorisation
        '
        Me.Timer_Automatic_Document_Categorisation.Enabled = True
        Me.Timer_Automatic_Document_Categorisation.Interval = 60000
        '
        'HelpProvider_Main
        '
        Me.HelpProvider_Main.HelpNamespace = "C:\Merrion Computing\Development\Projects\Printer Monitor\Source\FrontEnd\Printer" & _
            "MonitorServiceConfiguration\PrinterMonitoringViewer\Documentation\PrinterMonitor" & _
            "ingViewer.chm"
        '
        'MenuItem_File_Options
        '
        Me.MenuItem_File_Options.Index = 2
        Me.MenuItem_File_Options.Text = "&Options"
        '
        'Form_PrinterUsageViewer
        '
        Me.AutoScaleBaseSize = New System.Drawing.Size(5, 13)
        Me.ClientSize = New System.Drawing.Size(672, 441)
        Me.Controls.Add(Me.ListView_Sections)
        Me.Controls.Add(Me.Panel_Data)
        Me.Controls.Add(Me.Panel_Parameters)
        Me.HelpProvider_Main.SetHelpNavigator(Me, System.Windows.Forms.HelpNavigator.Find)
        Me.Icon = CType(resources.GetObject("$this.Icon"), System.Drawing.Icon)
        Me.Menu = Me.MainMenu_PrinterUsageViewer
        Me.MinimumSize = New System.Drawing.Size(300, 200)
        Me.Name = "Form_PrinterUsageViewer"
        Me.HelpProvider_Main.SetShowHelp(Me, True)
        Me.Text = "Printer Usage Viewer"
        Me.Panel_Parameters.ResumeLayout(False)
        CType(Me.PictureBox_Parameters, System.ComponentModel.ISupportInitialize).EndInit()
        Me.Panel_Data.ResumeLayout(False)
        CType(Me.DataGrid_Report, System.ComponentModel.ISupportInitialize).EndInit()
        Me.ResumeLayout(False)

    End Sub

#End Region

#Region "Menu click handlers"

    Private Sub MenuItem_Help_About_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Help_About.Click
        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("MenuItem_Help_About_Click")
        End If
        Dim fShow As New Form2_About
        fShow.ShowDialog()
    End Sub

    Private Sub MenuItem_File_Close_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_File_Close.Click
        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("MenuItem_File_Close_Click")
        End If
        Me.Close()
    End Sub

    Private Sub MenuItem_Filter_Printer(ByVal sender As Object, ByVal e As System.EventArgs)

        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("MenuItem_Filter_Printer_Click")
        End If

        If DataGrid_Report.DataSource Is Nothing Then
            Exit Sub
        End If

        With CType(sender, MenuItem)
            .Checked = Not .Checked
            Try
                If .Checked Then
                    _PrinterRestriction.AddItem(.Text)
                Else
                    _PrinterRestriction.RemoveItem(.Text)
                End If
            Catch ex As Exception
                If PrinterMonitorViewerApplication.ApplicationTracing.TraceWarning Then
                    Trace.WriteLine(ex.Message, "Filter Printer")
                End If
            End Try
            Call RefreshDataView()
            PrintersFiltered = True
        End With
    End Sub

    Private Sub MenuItem_Filter_User(ByVal sender As Object, ByVal e As System.EventArgs)

        If DataGrid_Report.DataSource Is Nothing Then
            Exit Sub
        End If

        With CType(sender, MenuItem)
            .Checked = Not .Checked
            Try
                If .Checked Then
                    _UserRestriction.AddItem(.Text)
                Else
                    _UserRestriction.RemoveItem(.Text)
                End If
            Catch ex As Exception
                If PrinterMonitorViewerApplication.ApplicationTracing.TraceWarning Then
                    Trace.WriteLine(ex.Message, "Filter User")
                End If
            End Try
            Call RefreshDataView()
        End With
        UsersFiltered = True

    End Sub

    '\\ --[MenuItem_Report_Selection]----------------------------------------------------------
    '\\ Called when the user selects one of the reports from the PrinterUsageReports collection
    '\\ ---------------------------------------------------------------------------------------
    Private Sub MenuItem_Report_Selection(ByVal sender As Object, ByVal e As System.EventArgs)

        Dim sReport As String
        With CType(sender, MenuItem)
            sReport = .Text
        End With

        Label_SelectedReport.Text = sReport
        SelectedReport = _Reports.Item(sReport)


        '\\ Allow the report to be refreshed
        MenuItem_Report_Refresh.Enabled = True

        '\\ Recreate the context menus (users, printers)
        Call CreatePrintersContextMenu()
        Call CreateUsersContextmenu()

    End Sub

    Private Sub MenuItem_Printers_ShowAll_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Printers_ShowAll.Click
        _PrinterRestriction.Clear()
        Call RefreshDataView()
        PrintersFiltered = False
    End Sub

    Private Sub MenuItem_Users_ShowAll_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Users_ShowAll.Click
        _UserRestriction.Clear()
        Call RefreshDataView()
        UsersFiltered = False
    End Sub

#Region "Date range menu items"

    Private Sub MenuItem_DateReange_All_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DateReange_All.Click
        With _DateRestriction
            .UnrestrictedEnd = True
            .UnrestrictedStart = True
        End With
        Call RefreshDataView()
        MenuItem_DateReange_All.Checked = True
        MenuItem_DateRange_Today.Checked = False
        MenuItem_DateRange_ThisWeek.Checked = False
        MenuItem_DateRange_ThisMonth.Checked = False
        MenuItem_DateRange_Custom.Checked = False
        DaterangeFiltered = False
    End Sub

    Private Sub MenuItem_DateRange_Today_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DateRange_Today.Click
        With _DateRestriction
            .UnrestrictedEnd = True
            .StartDate = Date.Today
        End With
        Call RefreshDataView()
        MenuItem_DateReange_All.Checked = False
        MenuItem_DateRange_Today.Checked = True
        MenuItem_DateRange_ThisWeek.Checked = False
        MenuItem_DateRange_ThisMonth.Checked = False
        MenuItem_DateRange_Custom.Checked = False
        DaterangeFiltered = True
    End Sub

    Private Sub MenuItem_DateRange_ThisWeek_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DateRange_ThisWeek.Click
        With _DateRestriction
            .UnrestrictedEnd = True
            .StartDate = DateAdd(DateInterval.Day, -7, Date.Today)
        End With
        Call RefreshDataView()
        MenuItem_DateReange_All.Checked = False
        MenuItem_DateRange_Today.Checked = False
        MenuItem_DateRange_Custom.Checked = False
        MenuItem_DateRange_ThisMonth.Checked = False
        MenuItem_DateRange_ThisWeek.Checked = True
        DaterangeFiltered = True
    End Sub

    Private Sub MenuItem_DateRange_ThisMonth_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DateRange_ThisMonth.Click
        With _DateRestriction
            .UnrestrictedEnd = True
            .StartDate = DateAdd(DateInterval.Month, -1, Date.Today)
        End With
        Call RefreshDataView()
        MenuItem_DateReange_All.Checked = False
        MenuItem_DateRange_Today.Checked = False
        MenuItem_DateRange_ThisWeek.Checked = False
        MenuItem_DateRange_Custom.Checked = False
        MenuItem_DateRange_ThisMonth.Checked = True
        DaterangeFiltered = True
    End Sub

    Private Sub MenuItem_DateRange_Custom_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DateRange_Custom.Click
        Dim frmDateRestriction As New Form_CustomDateRange(_DateRestriction)
        If frmDateRestriction.ShowDialog(Me) = DialogResult.OK Then
            _DateRestriction = frmDateRestriction.Restriction
            Call RefreshDataView()
            MenuItem_DateReange_All.Checked = False
            MenuItem_DateRange_Today.Checked = False
            MenuItem_DateRange_ThisWeek.Checked = False
            MenuItem_DateRange_ThisMonth.Checked = False
            MenuItem_DateRange_Custom.Checked = True
            DaterangeFiltered = Not (_DateRestriction.Restriction = "")
        End If
    End Sub
#End Region

    Private Sub MenuItem_Report_Refresh_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Report_Refresh.Click
        SelectedReport = _SelectedReport
    End Sub

    Private Sub MenuItem_DocumentName_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DocumentName.Click
        Dim fDocFilter As New Form_DocumentNameRestriction(_DocumentRestriction)
        With fDocFilter
            .TextBox_DocumentName.Text = _DocumentRestriction.DocumentName
            .ShowDialog(Me)
            If .DialogResult = DialogResult.OK Then
                _DocumentRestriction.DocumentName = .TextBox_DocumentName.Text
                _DocumentRestriction.Modifier = CType(.ComboBox_RestrictionType.SelectedIndex, DataViewDocumentRestriction.RestrictionModifier)
                Call RefreshDataView()
                DocumentsFiltered = Not (_DocumentRestriction.Restriction = "")
            End If
        End With
    End Sub

    Private Sub MenuItem_File_PageSetup_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_File_PageSetup.Click

        If _Printer Is Nothing Then
            _Printer = New DataGridPrinter(Me.DataGrid_Report)
        End If
        '\\ Set the document to which the page setup will be applied...
        PageSetupDialogMain.Document = _Printer.PrintDocument
        If PageSetupDialogMain.ShowDialog = DialogResult.OK Then
            '\\ New page settings are applied to _GridPrintDocument
        End If

    End Sub

    Private Sub MenuItem_Report_Save_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Report_Save.Click

        If Me.SaveFileDialog_Report.ShowDialog = DialogResult.OK Then
            Call SaveReport(SaveFileDialog_Report.FileName, SaveFileDialog_Report.FilterIndex)
        End If

    End Sub

    Private Sub MenuItem_Document_Categories_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Document_Categories.Click

        Dim fDialog As New Form_DocumentCategories
        fDialog.Show()
        fDialog.BringToFront()

    End Sub


    Private Sub MenuItem_File_Print_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_File_Print.Click

        'If _GridPrintDocument Is Nothing Then
        '_GridPrintDocument = New PrintDocument
        'End If


        '\\ If there is nothing to print, just ignore the click
        If Me.DataGrid_Report.DataSource Is Nothing Then
            Exit Sub
        End If

        If _Printer Is Nothing Then
            _Printer = New DataGridPrinter(Me.DataGrid_Report)
        Else
            _Printer.DataGrid = Me.DataGrid_Report
        End If

        _Printer.HeaderText = Me.Label_SelectedReport.Text
        _Printer.HeaderFont = New System.Drawing.Font(System.Drawing.FontFamily.GenericSansSerif, 20)

        With PrintPreviewDialog_Report
            .Document = _Printer.PrintDocument
            If .ShowDialog = DialogResult.OK Then
                '\\ Print the document for real...
                _Printer.Print()
            End If
            '\\ Clear out the document from the preview pane
            .Document = Nothing
        End With

    End Sub

#Region "DataGrid context menu"
    Private Sub MenuItem_DataGrid_Column_Settings_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_DataGrid_Column_Settings.Click

        Dim fColumns As New Form_DataGrid_Columns
        Try
            fColumns.DataGrid = Me.DataGrid_Report
            Call fColumns.ShowDialog()
            '\\ Apply the new style to the table
        Catch ex As Exception
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.ToString, Me.GetType.ToString)
            End If
        End Try


    End Sub
#End Region

#End Region

#Region "Private methods"
#Region "ExecuteQuery"
    '\\ --[ExecuteQuery]----------------------------------
    '\\ Runs the SQL query and updates the data grid
    '\\ --------------------------------------------------
    Private Sub ExecuteQuery(ByVal SQL As String)

        If _DBConn.State = ConnectionState.Closed Then
            _DBConn.Open()
        End If
        Dim sqlCmd As New SqlCommand(SQL, _DBConn)
        Dim adapter As New SqlDataAdapter(sqlCmd)
        dt.Clear()
        dt = New DataTable
        adapter.Fill(dt)

        DataGrid_Report.SetDataBinding(dt, "")
        Call RefreshDataView()

    End Sub

#End Region

#Region "CreateContextMenus"
    '\\ --[CreateContextMenus]----------------------------
    '\\ Creates the dynamic "Printers" and "Users" context 
    '\\ menus
    '\\ ---------------------------------------------------
    Private Sub CreatePrintersContextMenu()

        If _DBConn.State = ConnectionState.Closed Then
            _DBConn.Open()
        End If

        '\\ Get the list of printers for the printers context menu
        Dim sqlcmdPrinters As New SqlCommand("SELECT DISTINCT Name FROM PRINTERS Order By Name", _DBConn)
        Dim PrintersReader As SqlDataReader = sqlcmdPrinters.ExecuteReader
        MenuItem_Printers_Filter.MenuItems.Clear()
        While PrintersReader.Read
            MenuItem_Printers_Filter.MenuItems.Add(PrintersReader.GetString(0), AddressOf MenuItem_Filter_Printer)
        End While
        PrintersReader.Close()

        '\\ Close the DB connection to tidy it up
        _DBConn.Close()

    End Sub

    Public Sub CreateUsersContextmenu()

        If _DBConn.State = ConnectionState.Closed Then
            _DBConn.Open()
        End If

        '\\ Get the list of users for the users context menu
        Dim sqlcmdUsers As New SqlCommand("SELECT DISTINCT  [User Name] FROM [Print Jobs] Order By [User Name]", _DBConn)
        Dim UsersReader As SqlDataReader = sqlcmdUsers.ExecuteReader
        MenuItem_Users_Filter.MenuItems.Clear()
        Dim lMenuCount As Integer
        While UsersReader.Read
            lMenuCount += 1
            Dim miU As MenuItem = MenuItem_Users_Filter.MenuItems.Add(UsersReader.GetString(0), AddressOf MenuItem_Filter_User)
            If lMenuCount Mod 20 = 0 Then
                miU.Break = True
            End If
        End While
        UsersReader.Close()

        '\\ Close the DB connection to tidy it up
        _DBConn.Close()

    End Sub
#End Region

#Region "SelectedReport"
    Private WriteOnly Property SelectedReport() As PrinterUsageReport
        Set(ByVal Value As PrinterUsageReport)
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine(Value.Name, "SelectedReport")
            End If
            _SelectedReport = Value
            With _SelectedReport
                Me.MenuItem_DateRange_Custom.Enabled = .CanRestrictByDateRange
                Me.MenuItem_DateRange_ThisMonth.Enabled = .CanRestrictByDateRange
                Me.MenuItem_DateRange_ThisWeek.Enabled = .CanRestrictByDateRange
                Me.MenuItem_DateRange_Today.Enabled = .CanRestrictByDateRange
                Me.MenuItem_DocumentName.Enabled = .CanRestrictByDocumentName
                Me.MenuItem_Printers_Filter.Enabled = .CanRestrictByPrinter
                Me.MenuItem_Users_Filter.Enabled = .CanRestrictByUser
            End With
            Try
                With _SelectedReport
                    Dim sSQL As String
                    sSQL = .QueryDef.ToUpper
                    Call Me.ExecuteQuery(sSQL)
                    Call RefreshDataView()
                End With
            Catch ed As Exception
                If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                    Trace.WriteLine(ed.Message, Me.GetType.ToString)
                End If
            End Try
        End Set
    End Property
#End Region

    Private Sub ContextMenu_Printers_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles ContextMenu_Printers.Popup
        Call CreatePrintersContextMenu()
    End Sub


    Private Sub ContextMenu_Users_Popup(ByVal sender As Object, ByVal e As System.EventArgs) Handles ContextMenu_Users.Popup
        Call CreateUsersContextmenu()
    End Sub

#Region "MenuItem_Application_Settings"
    Private Sub MenuItem_Application_Settings_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles MenuItem_Application_Settings.Click
        Dim fDialog As New Form_ApplicationSettings
        fDialog.ShowDialog(Me)
    End Sub
#End Region

#End Region

#Region "ListView_Sections events"
    Private Sub ListView_Sections_SelectedIndexChanged(ByVal sender As Object, ByVal e As System.EventArgs) Handles ListView_Sections.SelectedIndexChanged
        If ListView_Sections.SelectedItems.Count > 0 Then
            If ListView_Sections.SelectedItems(0).Text = "Printers" Then
                ListView_Sections.ContextMenu = ContextMenu_Printers
            ElseIf ListView_Sections.SelectedItems(0).Text = "Users" Then
                ListView_Sections.ContextMenu = ContextMenu_Users
            ElseIf ListView_Sections.SelectedItems(0).Text = "Documents" Then
                ListView_Sections.ContextMenu = ContextMenu_Documents
            ElseIf ListView_Sections.SelectedItems(0).Text = "Date Range" Then
                ListView_Sections.ContextMenu = ContextMenu_DateRange
            Else '\\ settings
                ListView_Sections.ContextMenu = ContextMenu_Settings
            End If
        End If
    End Sub


#End Region

#Region "RefreshDataView"

    Private Sub RefreshDataView()

        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("RefreshDataView")
        End If

        If DataGrid_Report.DataSource Is Nothing Then
            Exit Sub
        End If

        Dim sRestriction As String
        If Not _PrinterRestriction.Unrestricted Then
            sRestriction = _PrinterRestriction.Restriction
            If Not _UserRestriction.Unrestricted OrElse Not (_DateRestriction.UnrestrictedStart And _DateRestriction.UnrestrictedEnd) Then
                sRestriction = sRestriction & " AND "
            End If
        End If

        If _OnlyOwnDocuments Then
            '\\ Add the user name restriction to the report
            _UserRestriction.Clear()
            _UserRestriction.AddItem(_LoggedInUsername)
        End If

        If (Not _UserRestriction.Unrestricted) Then
            sRestriction = sRestriction & _UserRestriction.Restriction
            If Not (_DateRestriction.UnrestrictedStart And _DateRestriction.UnrestrictedEnd) Then
                sRestriction = sRestriction & " AND "
            End If
        End If
        If Not (_DateRestriction.UnrestrictedStart And _DateRestriction.UnrestrictedEnd) Then
            sRestriction = sRestriction & _DateRestriction.Restriction
        End If
        If Not _DocumentRestriction.Modifier = DataViewDocumentRestriction.RestrictionModifier.None Then
            If sRestriction <> "" Then
                sRestriction = sRestriction & " AND "
            End If
            sRestriction = sRestriction & _DocumentRestriction.Restriction
        End If

        dt.DefaultView.RowFilter = sRestriction

        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("RefreshDataView.Restriction = " & sRestriction)
        End If

        '\\ Set up the TableStyles property
        With DataGrid_Report.TableStyles
            .Clear()
            .Add(New DataGridTableStyle(True))
        End With

    End Sub
#End Region

#Region "Button events"
    Private Sub Button_SelectReport_Click(ByVal sender As Object, ByVal e As System.EventArgs) Handles Button_SelectReport.Click
        '\\ Show the list of reports 
        ContextMenu_Reports.Show(Label_SelectedReport, New Point(Label_SelectedReport.Width - 75, 2))
    End Sub
#End Region

#Region "OnlyOwnDocuments"
    Private WriteOnly Property OnlyOwnDocuments() As Boolean
        Set(ByVal Value As Boolean)
            Static _DoneOnce As Boolean
            _OnlyOwnDocuments = Value
            If _OnlyOwnDocuments AndAlso Not _DoneOnce Then
                _DoneOnce = True
                ListView_Sections.Items(ListViewSections.Setup).Remove()
                ListView_Sections.Items(ListViewSections.Users).Remove()
            End If
        End Set
    End Property
#End Region

#Region "SaveReport"
    Private Sub SaveReport(ByVal Filename As String, ByVal FileType As SaveFileDialogReportFilterIndexes)

        If PrinterMonitorViewerApplication.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("SaveReport(" & Filename & "," & FileType.ToString & ")", Me.GetType.ToString)
        End If

        Dim fiOut As New System.IO.FileInfo(Filename)
        '\\ Warn before overwriting existing file
        If fiOut.Exists Then
            If MessageBox.Show("File " & Filename & " already exists. Overwrite?", "Save Report", MessageBoxButtons.YesNo, MessageBoxIcon.Question) = DialogResult.No Then
                Exit Sub
            End If
        End If

        If FileType = SaveFileDialogReportFilterIndexes.SaveAs_Tab Then
            Call SaveReportTab(fiOut)
        Else
            Call SaveReportXML(fiOut)
        End If

    End Sub

    Private Sub SaveReportXML(ByVal fiOut As System.IO.FileInfo)

        '\\ Open the file for writing..
        Dim XMLFileWriter As System.Xml.XmlTextWriter
        XMLFileWriter = New Xml.XmlTextWriter(fiOut.FullName, System.Text.Encoding.UTF8)

        '\\ Get the data from the data grid...System.Data.DataTable
        Dim nCurrentRow As Integer
        Dim nColumn As Integer
        Dim Items() As Object

        With CType(Me.DataGrid_Report.DataSource, System.Data.DataTable)
            XMLFileWriter.WriteStartDocument()
            '\\ Add the additional comments
            XMLFileWriter.WriteComment("Report : " & Me.Label_SelectedReport.Text)
            XMLFileWriter.WriteComment("Date Written : " & Now.ToString)
            XMLFileWriter.WriteStartElement("PrinterUsageReport")
            For nCurrentRow = 0 To .DefaultView.Count - 1
                XMLFileWriter.WriteStartElement("ReportRow")
                XMLFileWriter.WriteAttributeString("RowNumber", nCurrentRow.ToString)
                Items = .DefaultView.Item(nCurrentRow).Row.ItemArray
                For nColumn = 0 To Items.Length - 1
                    XMLFileWriter.WriteAttributeString(StripSpaces(.Columns(nColumn).ToString), Items(nColumn).ToString)
                Next
                '\\ Next row
                XMLFileWriter.WriteEndElement()
            Next
            XMLFileWriter.WriteComment("(c) 2005 - Merrion Computing Ltd")
            XMLFileWriter.WriteEndElement()
            XMLFileWriter.WriteEndDocument()
        End With

        XMLFileWriter.Close()

    End Sub

    Private Sub SaveReportTab(ByVal fiOut As System.IO.FileInfo)

        '\\ Open the file for writing..
        Dim TabFileWriter As System.IO.StreamWriter
        TabFileWriter = New System.IO.StreamWriter(fiOut.FullName)

        '\\ Get the data from the data grid...System.Data.DataTable
        Dim nCurrentRow As Integer
        Dim nColumn As Integer
        Dim Items() As Object
        With CType(Me.DataGrid_Report.DataSource, System.Data.DataTable)
            For nColumn = 0 To .Columns.Count - 1
                TabFileWriter.Write(.Columns(nColumn).ToString & Chr(9))
            Next
            '\\ Next row
            TabFileWriter.Write(TabFileWriter.NewLine)
            For nCurrentRow = 0 To .DefaultView.Count - 1
                Items = .DefaultView.Item(nCurrentRow).Row.ItemArray
                For nColumn = 0 To Items.Length - 1
                    TabFileWriter.Write(Items(nColumn).ToString & Chr(9))
                Next
                '\\ Next row
                TabFileWriter.Write(TabFileWriter.NewLine)
            Next
        End With

        TabFileWriter.Close()


    End Sub
#End Region

#Region "Timer_Automatic_Document_Categorisation"
    Private Sub Timer_Automatic_Document_Categorisation_Tick(ByVal sender As Object, ByVal e As System.EventArgs) Handles Timer_Automatic_Document_Categorisation.Tick

        If _DBConn.State = ConnectionState.Closed Then
            _DBConn.Open()
        End If

        Dim sql As String
        sql = "EXECUTE Automated_Document_Categorisation "
        Dim cmd As New SqlCommand(sql, _DBConn)
        Try
            cmd.ExecuteNonQuery()
        Catch ex As Exception
            If PrinterMonitorViewerApplication.ApplicationTracing.TraceError Then
                Trace.WriteLine(ex.Message, Me.GetType.ToString)
            End If
        End Try

        _DBConn.Close()

    End Sub
#End Region

#Region "Filtered Status Properties"

    Public WriteOnly Property PrintersFiltered() As Boolean
        Set(ByVal Value As Boolean)
            If Value Then
                ListView_Sections.Items(ListViewSections.Printers).ImageIndex = ListViewImages.PrintersFiltered
            Else
                ListView_Sections.Items(ListViewSections.Printers).ImageIndex = ListViewImages.Printers
            End If
        End Set
    End Property

    Private WriteOnly Property DocumentsFiltered() As Boolean
        Set(ByVal Value As Boolean)
            If Value Then
                ListView_Sections.Items(ListViewSections.Documents).ImageIndex = ListViewImages.DocumentsFilterd
            Else
                ListView_Sections.Items(ListViewSections.Documents).ImageIndex = ListViewImages.Documents
            End If
        End Set
    End Property

    Private WriteOnly Property UsersFiltered() As Boolean
        Set(ByVal Value As Boolean)
            If Value Then
                ListView_Sections.Items(ListViewSections.Users).ImageIndex = ListViewImages.UsersFiltered
            Else
                ListView_Sections.Items(ListViewSections.Users).ImageIndex = ListViewImages.Users
            End If
        End Set
    End Property

    Private WriteOnly Property DaterangeFiltered() As Boolean
        Set(ByVal Value As Boolean)
            If Value Then
                ListView_Sections.Items(ListViewSections.DateRange).ImageIndex = ListViewImages.TimerangeFiltered
            Else
                ListView_Sections.Items(ListViewSections.DateRange).ImageIndex = ListViewImages.Timerange
            End If
        End Set
    End Property
#End Region

End Class

