'\\ --[CustomConfigurationSectionReaders]----------------
'\\ Special custom configuration section readers for the
'\\ PrinterMonitorService application
'\\ (c) 2004 Merrion Computing Ltd
'\\ -----------------------------------------------------
Imports System.Configuration
Imports System.Xml

Namespace CustomConfigurationSectionReaders

#Region "ListenerTypes enumeration"
    Public Enum ListenerTypes
        PrinterEventListener = 1
        PrintJobEventListener = 2
    End Enum
#End Region

#Region "PrinterEventListenersSectionReader"
    Public NotInheritable Class PrinterEventListenersSectionReader
        Implements IConfigurationSectionHandler

#Region "Private constants"
        Private Const SECTION_NAME As String = "printerEventListeners"
        Private Const ATTRIBUTE_NAME As String = "printerEventListener"
#End Region

        Shared Sub New()
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Configuration reader for ", SECTION_NAME)
            End If
            ConfigurationSettings.GetConfig(SECTION_NAME)
        End Sub

        Public Shared Function GetPrinterEventListenersNamedListenerDictionary() As NamedListenerDictionary
            Return CType(System.Configuration.ConfigurationSettings.GetConfig(SECTION_NAME), NamedListenerDictionary)
        End Function

#Region "IConfigurationSectionHandler interface"
        Public Function Create( _
                 ByVal parent As Object, _
                 ByVal configContext As Object, _
                 ByVal section As XmlNode _
                 ) As Object Implements IConfigurationSectionHandler.Create

            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Create ", Me.GetType.ToString)
            End If

            Dim retVal As New NamedListenerDictionary()

            If section.HasChildNodes Then
                Dim childNode As XmlNode
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.Indent()
                End If
                For Each childNode In section.ChildNodes
                    If childNode.Name = ATTRIBUTE_NAME Then
                        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                            Trace.WriteLine(childNode.InnerText, Me.GetType.ToString)
                        End If
                        Dim NamedPrinterListener As New NamedListener(childNode.Attributes.ItemOf("key").Value, ListenerTypes.PrinterEventListener, childNode.Attributes.ItemOf("classType").Value, childNode.Attributes.ItemOf("commandLine").Value)

                        retVal.Add(NamedPrinterListener.Key, NamedPrinterListener)
                    Else
                        If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                            Trace.WriteLine("Unexpected node : " & childNode.Name, Me.GetType.ToString)
                        End If
                    End If
                Next
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.Unindent()
                End If
            End If

            Return retVal

        End Function
#End Region

        Private Sub New()
            '\\ make this class non-creatable
        End Sub
    End Class
#End Region

#Region "PrintJobEventListenersSectionReader"
    Public NotInheritable Class PrintJobEventListenersSectionReader
        Implements IConfigurationSectionHandler

#Region "Private constants"
        Private Const SECTION_NAME As String = "printJobEventListeners"
        Private Const ATTRIBUTE_NAME As String = "printJobEventListener"
#End Region


        Shared Sub New()
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Configuration reader for ", SECTION_NAME)
            End If
            ConfigurationSettings.GetConfig(SECTION_NAME)
        End Sub

        Public Shared Function GetPrintJobNamedListenerDictionary() As NamedListenerDictionary
            Return CType(System.Configuration.ConfigurationSettings.GetConfig(SECTION_NAME), NamedListenerDictionary)
        End Function

#Region "IConfigurationSectionHandler interface"
        Public Function Create( _
                 ByVal parent As Object, _
                 ByVal configContext As Object, _
                 ByVal section As XmlNode _
                 ) As Object Implements IConfigurationSectionHandler.Create

            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Create ", Me.GetType.ToString)
            End If

            Dim retVal As New NamedListenerDictionary()

            If section.HasChildNodes Then
                Dim childNode As XmlNode
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.Indent()
                End If
                For Each childNode In section.ChildNodes
                    If childNode.Name = ATTRIBUTE_NAME Then
                        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                            Trace.WriteLine(childNode.InnerText, Me.GetType.ToString)
                        End If
                        Dim NamedPrinterListener As New NamedListener(childNode.Attributes.ItemOf("key").Value, ListenerTypes.PrintJobEventListener, childNode.Attributes.ItemOf("classType").Value, childNode.Attributes.ItemOf("commandLine").Value)
                        retVal.Add(NamedPrinterListener.Key, NamedPrinterListener)
                    Else
                        If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                            Trace.WriteLine("Unexpected node : " & childNode.Name, Me.GetType.ToString)
                        End If
                    End If
                Next
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.Unindent()
                End If
            End If

            Return retVal
        End Function
#End Region

        Private Sub New()
            '\\ make this class non creatable
        End Sub

    End Class
#End Region

#Region "NamedListener"
    '\\ --[NamedListener]--------------------------------
    '\\ A named listener describes a 
    Public Class NamedListener

#Region "Private member variables"
        Private _Key As String
        Private _ListenerType As ListenerTypes
        Private _ImplementingClassType As String
        Private _CommandLine As String
        Private _Instance As Object
#End Region

#Region "Public properties"

#Region "Key"
        Public ReadOnly Property Key() As String
            Get
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Key property get", Me.GetType.ToString)
                End If
                Return _Key
            End Get
        End Property
#End Region

#Region "ListenerType"
        Public ReadOnly Property ListenerType() As ListenerTypes
            Get
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("ListenerType property get", Me.GetType.ToString)
                End If
                Return _ListenerType
            End Get
        End Property
#End Region

#Region "ImplementingClassType"
        Public ReadOnly Property ImplementingClassType() As Type
            Get
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("ImplementingClassType property get", Me.GetType.ToString)
                End If
                Return Type.GetType(_ImplementingClassType, True, True)
            End Get
        End Property
#End Region

#Region "CommandLine"
        Public ReadOnly Property CommandLine() As String
            Get
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Commandline property get", Me.GetType.ToString)
                End If
                Return _CommandLine
            End Get
        End Property
#End Region

#Region "Instance"
        Public ReadOnly Property Instance() As Object
            Get
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Instance property get", Me.GetType.ToString)
                End If
                If _Instance Is Nothing Then
                    Try
                        _Instance = Activator.CreateInstance(Me.ImplementingClassType)
                        _Instance.CommandLine = _CommandLine
                    Catch e As Exception
                        If PrinterMonitoringService.ApplicationTracing.TraceError Then
                            Trace.WriteLine(e.ToString, Me.GetType.ToString)
                        End If
                    End Try
                End If
                Return _Instance
            End Get
        End Property
#End Region

#End Region

#Region "Public constructor"
        '\\ Parameters:
        '\\  Key = the unique value by which this listener is known
        '\\  ListenerTypeName = ["PrinterEventListener"|"PrintJobEventListener"]
        '\\  ClassTypeName = class implementing the listener e.g. "PrinterMonitorService.PrinterMonitorLogfileListener"
        '\\  CommandLine = startup parameters for the class, e.g. "/o MyPrinterEvents.log"
        Public Sub New(ByVal Key As String, ByVal ListenerType As ListenerTypes, ByVal ClassTypeName As String, ByVal CommandLine As String)

            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Constructor (" & Key & "," & ListenerType.ToString & "," & ClassTypeName & "," & CommandLine & ")", Me.GetType.ToString)
            End If
            _Key = Key
            _ListenerType = ListenerType
            _ImplementingClassType = ClassTypeName
            _CommandLine = CommandLine
        End Sub

#End Region

    End Class
#End Region

#Region "NamedListenerDictionary"
    ''' <summary>
    ''' A keyed dictionary collection of named listeners
    ''' </summary>
    ''' <remarks></remarks>
    Public Class NamedListenerDictionary
        Inherits Generic.Dictionary(Of String, NamedListener)


    End Class
#End Region

#Region "MonitoredPrintersSectionReader"
    ''' <summary>
    ''' Reads the "monitoredPrinters" section of the service .config file
    ''' </summary>
    ''' <remarks></remarks>
    Public NotInheritable Class MonitoredPrintersSectionReader
        Implements IConfigurationSectionHandler

#Region "Private constants"
        Private Const SECTION_NAME As String = "monitoredPrinters"
        Private Const ATTRIBUTE_NAME As String = "monitoredPrinter"
#End Region

        Shared Sub New()
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Configuration reader for ", SECTION_NAME)
            End If
            ConfigurationSettings.GetConfig(SECTION_NAME)
        End Sub

        Public Shared Function GetMonitoredPrintersCollection() As Collection
            Return CType(System.Configuration.ConfigurationSettings.GetConfig(SECTION_NAME), Collection)
        End Function

#Region "IConfigurationSectionHandler interface"
        Public Function Create( _
                 ByVal parent As Object, _
                 ByVal configContext As Object, _
                 ByVal section As XmlNode _
                 ) As Object Implements IConfigurationSectionHandler.Create

            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("Create ", Me.GetType.ToString)
            End If

            Dim retVal As New Collection()

            If section.HasChildNodes Then
                Dim childNode As XmlNode
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.Indent()
                End If
                For Each childNode In section.ChildNodes
                    If childNode.Name = ATTRIBUTE_NAME Then
                        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                            Trace.WriteLine(childNode.Attributes.ItemOf("DeviceName").Value, Me.GetType.ToString)
                        End If
                        retVal.Add(New MonitoredPrinter(childNode.Attributes.ItemOf("DeviceName").Value))
                    Else
                        If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                            Trace.WriteLine("Unexpected node : " & childNode.Name, Me.GetType.ToString)
                        End If
                    End If
                Next
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.Unindent()
                End If
            End If

            Return retVal
        End Function
#End Region

        Private Sub New()
            '\\ make this class non creatable
        End Sub
    End Class
#End Region

#Region "MonitoredPrinter"
    Public Class MonitoredPrinter

#Region "Private members"
        Private _DeviceName As String
#End Region

#Region "DeviceName"
        Public ReadOnly Property DeviceName() As String
            Get
                Return _DeviceName
            End Get
        End Property
#End Region

#Region "Public constructor"
        Public Sub New(ByVal DeviceName As String)
            _DeviceName = DeviceName
        End Sub
#End Region
    End Class
#End Region

End Namespace
