Imports System.Data.SqlClient
Imports PrinterMonitorListnerbase

'\\ --[PrinterMonitorDatabaseListener]--------------------
'\\ Monitors printer change events to an SQL server database
'\\ (c) 2004 Merrion Computing Ltd
'\\ -----------------------------------------------------
Public Class PrinterMonitorDatabaseListener
    Inherits PrinterMonitorListenerBase

#Region "Private constants"
    Private Const PrinterChangedStoredProcName As String = "OnPrinterInformationChanged"

    Private Const FIELDSIZE_PRINTERNAME As Integer = 220
    Private Const FIELDSIZE_LOCATION As Integer = 1024
    Private Const FIELDSIZE_COMMENT As Integer = 1024

#End Region

#Region "Private member variables"
    Private _ConnectionString As String
    Private _DBConn As SqlConnection
#End Region

#Region "Command line startup parameters"
    Public Overrides WriteOnly Property CommandLine() As String
        Set(ByVal Value As String)
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("CommandLine(" & Value & ")", Me.GetType.ToString)
            End If
            '\\ Start up is the OleDb connection string
            _ConnectionString = Value
            Try
                _DBConn = New SqlConnection(_ConnectionString)
                _DBConn.Open()
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Connected to " & _DBConn.Database.ToString, Me.GetType.ToString)
                End If
            Catch eConn As Exception
                Trace.WriteLine(eConn.Message)
            End Try
        End Set
    End Property
#End Region

#Region "OnPrinterInformationChanged"
    Public Overrides Sub OnPrinterInformationChanged(ByVal PrinterName As String, ByVal Location As String, ByVal Comment As String, ByVal JobCount As Integer, _
        ByVal IsReady As Boolean, ByVal IsDoorOpen As Boolean, _
        ByVal IsInError As Boolean, ByVal IsInitialising As Boolean, _
        ByVal IsAwaitingManualFeed As Boolean, ByVal IsOutOfToner As Boolean, _
        ByVal IsUnavailable As Boolean, ByVal IsOffline As Boolean, _
        ByVal IsOutOfMemory As Boolean, ByVal IsOutputBinFull As Boolean, _
        ByVal IsPaperJammed As Boolean, ByVal IsOutOfPaper As Boolean, _
        ByVal IsPaused As Boolean, ByVal IsDeletingJob As Boolean, _
        ByVal IsInPowerSave As Boolean, ByVal IsPrinting As Boolean, _
        ByVal IsWaitingOnUserIntervention As Boolean, ByVal IsWarmingUp As Boolean, _
        ByVal ServerName As String, _
        ByVal ShareName As String, _
        ByVal PortName As String, _
        ByVal DriverName As String, _
        ByVal SeperatorFilename As String, _
        ByVal PrintProcessor As String, _
        ByVal DefaultDataType As String, _
        ByVal Parameters As String)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnPrinterInformationChanged at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If

        Dim cmdPrinterInformationChanged As SqlCommand

        If _DBConn.State = ConnectionState.Closed Then
            _DBConn.Open()
        End If

        '\\ If the server name passed in is blank, use this machine name
        If ServerName = "" Then
            ServerName = _DBConn.WorkstationId
        End If

        Try
            cmdPrinterInformationChanged = _DBConn.CreateCommand()
        Catch e As Exception
            Trace.WriteLine(e.Message, Me.GetType.ToString)
            _DBConn.Close()
            Exit Sub
        End Try

        cmdPrinterInformationChanged.CommandText = PrinterChangedStoredProcName
        With cmdPrinterInformationChanged
            .CommandType = CommandType.StoredProcedure
            .Parameters.Add("@PrinterName", SqlDbType.VarChar, FIELDSIZE_PRINTERNAME).Value = PrinterName
            .Parameters.Add("@Location", SqlDbType.VarChar, FIELDSIZE_LOCATION).Value = Location
            .Parameters.Add("@Comment", SqlDbType.VarChar, FIELDSIZE_COMMENT).Value = Comment
            .Parameters.AddWithValue("@JobCount", JobCount)
            .Parameters.AddWithValue("@IsReady", IsReady)
            .Parameters.AddWithValue("@IsDoorOpen", IsDoorOpen)
            .Parameters.AddWithValue("@IsInError", IsInError)
            .Parameters.AddWithValue("@IsInitialising", IsInitialising)
            .Parameters.AddWithValue("@IsAwaitingManualFeed", IsAwaitingManualFeed)
            .Parameters.AddWithValue("@IsOutOfToner", IsOutOfToner)
            .Parameters.AddWithValue("@IsUnavailable", IsUnavailable)
            .Parameters.AddWithValue("@IsOffline", IsOffline)
            .Parameters.AddWithValue("@IsOutOfMemory", IsOutOfMemory)
            .Parameters.AddWithValue("@IsOutputBinFull", IsOutputBinFull)
            .Parameters.AddWithValue("@IsPaperJammed", IsPaperJammed)
            .Parameters.AddWithValue("@IsOutOfPaper", IsOutOfPaper)
            .Parameters.AddWithValue("@IsPaused", IsPaused)
            .Parameters.AddWithValue("@IsDeletingJob", IsDeletingJob)
            .Parameters.AddWithValue("@IsInPowerSave", IsInPowerSave)
            .Parameters.AddWithValue("@IsPrinting", IsPrinting)
            .Parameters.AddWithValue("@IsWaitingOnUserIntervention", IsWaitingOnUserIntervention)
            .Parameters.AddWithValue("@IsWarmingUp", IsWarmingUp)
            '\\ 1.0.8 parameters
            .Parameters.AddWithValue("@ServerName", ServerName)
            .Parameters.AddWithValue("@ShareName", ShareName)
            .Parameters.AddWithValue("@PortName", PortName)
            .Parameters.AddWithValue("@DriverName", DriverName)
            .Parameters.AddWithValue("@SeperatorFilename", SeperatorFilename)
            .Parameters.AddWithValue("@PrintProcessor", PrintProcessor)
            .Parameters.AddWithValue("@DefaultDataType", DefaultDataType)
            .Parameters.AddWithValue("@Parameters", Parameters)
            Try
                Dim RowsAffected As Integer
                RowsAffected = .ExecuteNonQuery()
            Catch edb As System.Data.SqlClient.SqlException
                Trace.WriteLine(edb.Message)
                If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                    Dim eSQL As SqlError
                    For Each eSQL In edb.Errors
                        Trace.WriteLine(eSQL.Message, eSQL.Procedure.ToString)
                    Next
                End If
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Dim pIn As SqlParameter
                    For Each pIn In .Parameters
                        Trace.WriteLine(pIn.ParameterName & "=" & pIn.Value.ToString & "(" & pIn.Size & ")")
                    Next
                End If
            Finally
                _DBConn.Close()
            End Try
        End With
    End Sub

#End Region

#Region "Public constructor"
    Public Sub New()
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("New ", Me.GetType.ToString)
        End If
    End Sub
#End Region

#Region "Service events"
    Public Overrides Sub OnStopService()

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnStopService", Me.GetType.ToString)
        End If

        If Not _DBConn Is Nothing Then
            If _DBConn.State = ConnectionState.Open Then
                _DBConn.Close()
            End If
        End If
    End Sub
#End Region

End Class


'\\ --[PrintJobMonitorDatabaseListener]---------------------
'\\ Monitors print job events to an SQL server database
'\\ (c) 2004 Merrion Computing Ltd
'\\ --------------------------------------------------------
Public Class PrintJobMonitorDatabaseListener
    Inherits PrintJobMonitorListenerBase

#Region "Private constants"
    Private Const JobAddedStoredProcName As String = "OnJobAdded"
    Private Const JobSetStoredProcName As String = "OnJobSet"
    Private Const JobWrittenStoredProcName As String = "OnJobWritten"
    Private Const JobDeletedStoredProcName As String = "OnJobDeleted"

    Private Const FIELDSIZE_PRINTERNAME As Integer = 220
    Private Const FIELDSIZE_USERNAME As Integer = 255
    Private Const FIELDSIZE_DOCUMENTNAME As Integer = 255
    Private Const FIELDSIZE_MACHINENAME As Integer = 255

#End Region

#Region "Private member variables"
    Private _ConnectionString As String
#End Region

#Region "Public delegates"
    Public Delegate Sub JobAddedEvent(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal PaperType As String, _
                                       ByVal Landscape As Boolean, ByVal Colour As Boolean, _
                                       ByVal PaperSource As String, ByVal PrinterResolution As String, _
                                       ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                       ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                       ByVal InError As Boolean, ByVal Offline As Boolean, _
                                       ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                       ByVal DataType As String, _
                                       ByVal PaperWidth As Integer, _
                                       ByVal PaperHeight As Integer, _
                                       ByVal PrinterResolutionX As Integer, _
                                       ByVal PrinterResolutionY As Integer, _
                                       ByVal NotifyUserName As String, _
                                       ByVal PrintProcessorName As String, _
                                       ByVal DriverName As String, _
                                       ByVal Parameters As String, _
                                       ByVal QueuedTime As Integer, _
                                       ByVal JobSize As Integer, ByVal JobId As Integer)


    Public Delegate Sub JobWrittenEvent(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                         ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                         ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                         ByVal InError As Boolean, ByVal Offline As Boolean, _
                                         ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                         ByVal QueuedTime As Integer, _
                                         ByVal JobSize As Integer, ByVal JobId As Integer)

    Public Delegate Sub JobDeletedEvent(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                         ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                         ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                         ByVal InError As Boolean, ByVal Offline As Boolean, _
                                         ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                         ByVal QueuedTime As Integer, _
                                         ByVal JobSize As Integer, ByVal JobId As Integer)

    Public Delegate Sub JobSetEvent(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                         ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                         ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                         ByVal InError As Boolean, ByVal Offline As Boolean, _
                                         ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                         ByVal QueuedTime As Integer, _
                                         ByVal JobSize As Integer, ByVal JobId As Integer)

#End Region

#Region "Command line startup parameters"
    Public Overrides WriteOnly Property CommandLine() As String
        Set(ByVal Value As String)
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("CommandLine(" & Value & ")", Me.GetType.ToString)
            End If
            '\\ Start up is the OleDb connection string
            _ConnectionString = Value
        End Set
    End Property
#End Region

#Region "OnJobAdded"
    Public Overrides Sub OnJobAdded(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal PaperType As String, _
                                       ByVal Landscape As Boolean, ByVal Colour As Boolean, _
                                       ByVal PaperSource As String, ByVal PrinterResolution As String, _
                                       ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                       ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                       ByVal InError As Boolean, ByVal Offline As Boolean, _
                                       ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                       ByVal DataType As String, _
                                       ByVal PaperWidth As Integer, _
                                       ByVal PaperHeight As Integer, _
                                       ByVal PrinterResolutionX As Integer, _
                                       ByVal PrinterResolutionY As Integer, _
                                       ByVal NotifyUserName As String, _
                                       ByVal PrintProcessorName As String, _
                                       ByVal DriverName As String, _
                                       ByVal Parameters As String, _
                                       ByVal QueuedTime As Integer, _
                                       ByVal JobSize As Integer, _
                                       ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobAdded at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If

        Dim JobAddedHandler As New JobAddedEvent(AddressOf AsynchJobAdded)
        JobAddedHandler.BeginInvoke(PrinterName, Username, Document, Machinename, Pages, Copies, Submitted, PaperType, _
                                     Landscape, Colour, _
                                     PaperSource, PrinterResolution, _
                                     Paused, Deleted, Deleting, _
                                     Printed, Printing, _
                                     InError, Offline, _
                                     PaperOut, UserInterventionRequired, _
                                     DataType, _
                                     PaperWidth, _
                                     PaperHeight, _
                                     PrinterResolutionX, _
                                     PrinterResolutionY, _
                                     NotifyUserName, _
                                     PrintProcessorName, _
                                     DriverName, _
                                     Parameters, _
                                     QueuedTime, _
                                     JobSize, _
                                     JobId, _
                                     New System.AsyncCallback(AddressOf AsynchJobAddComplete), Nothing)


    End Sub
#End Region

#Region "OnJobDeleted"
    Public Overrides Sub OnJobDeleted(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                         ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                         ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                         ByVal InError As Boolean, ByVal Offline As Boolean, _
                                         ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                         ByVal QueuedTime As Integer, ByVal JobSize As Integer, ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobDeleted at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If

        Dim JobDeletedHandler As New JobDeletedEvent(AddressOf AsynchJobDeleted)
        JobDeletedHandler.BeginInvoke(PrinterName, Username, Document, Machinename, Pages, Copies, _
                                      Submitted, Status, PagesPrinted, Paused, Deleted, Deleting, _
                                       Printed, Printing, InError, Offline, PaperOut, UserInterventionRequired, _
                                       QueuedTime, JobSize, JobId, _
                                       New System.AsyncCallback(AddressOf AsynchJobDeleteComplete), Nothing)


    End Sub
#End Region

#Region "OnJobSet"
    Public Overrides Sub OnJobSet(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                  ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                  ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                  ByVal InError As Boolean, ByVal Offline As Boolean, _
                                  ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                  ByVal QueuedTime As Integer, ByVal JobSize As Integer, ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobSet at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If

        Dim JobSetHandler As New JobSetEvent(AddressOf AsynchJobSet)
        JobSetHandler.BeginInvoke(PrinterName, Username, Document, Machinename, Pages, Copies, _
                                      Submitted, Status, PagesPrinted, Paused, Deleted, Deleting, _
                                       Printed, Printing, InError, Offline, PaperOut, UserInterventionRequired, _
                                       QueuedTime, JobSize, JobId, _
                                       New System.AsyncCallback(AddressOf AsynchJobSetComplete), Nothing)


    End Sub
#End Region

#Region "OnJobWritten"
    Public Overrides Sub OnJobWritten(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                     ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                     ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                     ByVal InError As Boolean, ByVal Offline As Boolean, _
                                     ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                     ByVal QueuedTime As Integer, ByVal JobSize As Integer, ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobWritten at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If

        Dim JobWrittenHandler As New JobWrittenEvent(AddressOf AsynchJobWritten)
        JobWrittenHandler.BeginInvoke(PrinterName, Username, Document, Machinename, Pages, Copies, _
                                      Submitted, Status, PagesPrinted, Paused, Deleted, Deleting, _
                                       Printed, Printing, InError, Offline, PaperOut, UserInterventionRequired, _
                                       QueuedTime, JobSize, JobId, _
                                       New System.AsyncCallback(AddressOf AsynchJobWrittenComplete), Nothing)


    End Sub
#End Region

#Region "Service events"
    Public Overrides Sub OnStopService()

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnStopService", Me.GetType.ToString)
        End If

    End Sub
#End Region

#Region "Delegate implementations"
#Region "AsynchJobAdded"
    Protected Sub AsynchJobAdded(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal PaperType As String, _
                                       ByVal Landscape As Boolean, ByVal Colour As Boolean, _
                                       ByVal PaperSource As String, ByVal PrinterResolution As String, _
                                       ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                       ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                       ByVal InError As Boolean, ByVal Offline As Boolean, _
                                       ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                       ByVal DataType As String, _
                                       ByVal PaperWidth As Integer, _
                                       ByVal PaperHeight As Integer, _
                                       ByVal PrinterResolutionX As Integer, _
                                       ByVal PrinterResolutionY As Integer, _
                                       ByVal NotifyUserName As String, _
                                       ByVal PrintProcessorName As String, _
                                       ByVal DriverName As String, _
                                       ByVal Parameters As String, _
                                       ByVal QueuedTime As Integer, _
                                       ByVal JobSize As Integer, ByVal JobId As Integer)


        Dim _DBConnAdd As New SqlConnection(_ConnectionString)

        If _DBConnAdd.State = ConnectionState.Closed Then
            _DBConnAdd.Open()
        End If

        Dim cmdPrintJobAdded As New SqlCommand(JobAddedStoredProcName, _DBConnAdd)
        With cmdPrintJobAdded
            .CommandType = CommandType.StoredProcedure
            .Parameters.Add("@PrinterName", SqlDbType.VarChar, FIELDSIZE_PRINTERNAME).Value = PrinterName
            .Parameters.Add("@Username", SqlDbType.VarChar, FIELDSIZE_USERNAME).Value = Username
            .Parameters.Add("@Document", SqlDbType.VarChar, FIELDSIZE_DOCUMENTNAME).Value = Document
            .Parameters.Add("@Machinename", SqlDbType.VarChar, FIELDSIZE_MACHINENAME).Value = Machinename
            .Parameters.Add("@Pages", SqlDbType.Int).Value = Pages
            .Parameters.Add("@Copies", SqlDbType.Int).Value = Copies
            .Parameters.Add("@Submitted", SqlDbType.DateTime).Value = Submitted
            .Parameters.Add("@PaperType", SqlDbType.VarChar).Value = PaperType
            .Parameters.Add("@JobId", SqlDbType.Int).Value = JobId
            .Parameters.Add("@Landscape", SqlDbType.Bit).Value = Landscape
            .Parameters.Add("@Colour", SqlDbType.Bit).Value = Colour
            .Parameters.Add("@PaperSource", SqlDbType.VarChar, 120).Value = PaperSource
            .Parameters.Add("@PrinterResolution", SqlDbType.VarChar, 100).Value = PrinterResolution
            .Parameters.Add("@Paused", SqlDbType.Bit).Value = Paused
            .Parameters.Add("@Deleted", SqlDbType.Bit).Value = Deleted
            .Parameters.Add("@Deleting", SqlDbType.Bit).Value = Deleting
            .Parameters.Add("@Printed", SqlDbType.Bit).Value = Printed
            .Parameters.Add("@Printing", SqlDbType.Bit).Value = Printing
            .Parameters.Add("@InError", SqlDbType.Bit).Value = InError
            .Parameters.Add("@Offline", SqlDbType.Bit).Value = Offline
            .Parameters.Add("@PaperOut", SqlDbType.Bit).Value = PaperOut
            .Parameters.AddWithValue("@UserInterventionRequired", UserInterventionRequired)
            .Parameters.AddWithValue("@DataType", DataType)
            .Parameters.AddWithValue("@PaperWidth", PaperWidth)
            .Parameters.AddWithValue("@PaperHeight", PaperHeight)
            .Parameters.AddWithValue("@PrinterResolutionX", PrinterResolutionX)
            .Parameters.AddWithValue("@PrinterResolutionY", PrinterResolutionY)
            .Parameters.AddWithValue("@NotifyUserName", NotifyUserName)
            .Parameters.AddWithValue("@PrintProcessorName", PrintProcessorName)
            .Parameters.AddWithValue("@DriverName", DriverName)
            .Parameters.AddWithValue("@Parameters", Parameters)
            .Parameters.AddWithValue("@QueuedTime", QueuedTime)
            .Parameters.AddWithValue("@JobSize", JobSize)
            Try
                Dim RowsAffected As Integer
                RowsAffected = .ExecuteNonQuery()
            Catch edb As System.Data.SqlClient.SqlException
                Trace.WriteLine(edb.Message)
                If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                    Dim eSQL As SqlError
                    For Each eSQL In edb.Errors
                        Trace.WriteLine(eSQL.Message, eSQL.Procedure.ToString)
                    Next
                End If
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Dim pIn As SqlParameter
                    For Each pIn In .Parameters
                        Trace.WriteLine(pIn.ParameterName & "=" & pIn.Value.ToString & "(" & pIn.Size & ")")
                    Next
                End If
            Finally
                _DBConnAdd.Close()
            End Try
        End With
    End Sub
#End Region

#Region "AsynchJobDeleted"
    Protected Sub AsynchJobDeleted(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                         ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                         ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                         ByVal InError As Boolean, ByVal Offline As Boolean, _
                                         ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                         ByVal QueuedTime As Integer, _
                                         ByVal JobSize As Integer, ByVal JobId As Integer)


        Dim _DBConnDelete As New SqlConnection(_ConnectionString)

        If _DBConnDelete.State = ConnectionState.Closed Then
            _DBConnDelete.Open()
        End If

        Dim cmdPrintJobDeleted As New SqlCommand(JobDeletedStoredProcName, _DBConnDelete)
        With cmdPrintJobDeleted
            .CommandType = CommandType.StoredProcedure
            .Parameters.Add("@PrinterName", SqlDbType.VarChar, FIELDSIZE_PRINTERNAME).Value = PrinterName
            .Parameters.Add("@Username", SqlDbType.VarChar, FIELDSIZE_USERNAME).Value = Username
            .Parameters.Add("@Document", SqlDbType.VarChar, FIELDSIZE_DOCUMENTNAME).Value = Document
            .Parameters.Add("@Machinename", SqlDbType.VarChar, FIELDSIZE_MACHINENAME).Value = Machinename
            .Parameters.Add("@Pages", SqlDbType.Int).Value = Pages
            .Parameters.Add("@Copies", SqlDbType.Int).Value = Copies
            .Parameters.Add("@Submitted", SqlDbType.DateTime).Value = Submitted
            .Parameters.AddWithValue("@Status", Status)
            .Parameters.AddWithValue("@PagesPrinted", PagesPrinted)
            .Parameters.AddWithValue("@Paused", Paused)
            .Parameters.AddWithValue("@Deleted", Deleted)
            .Parameters.AddWithValue("@Deleting", Deleting)
            .Parameters.AddWithValue("@Printed", Printed)
            .Parameters.AddWithValue("@Printing", Printing)
            .Parameters.AddWithValue("@InError", InError)
            .Parameters.AddWithValue("@Offline", Offline)
            .Parameters.AddWithValue("@PaperOut", PaperOut)
            .Parameters.AddWithValue("@UserInterventionRequired", UserInterventionRequired)
            .Parameters.AddWithValue("@QueuedTime", QueuedTime)
            .Parameters.AddWithValue("@JobSize", JobSize)
            .Parameters.AddWithValue("@JobId", JobId)
            Try
                Dim RowsAffected As Integer
                RowsAffected = .ExecuteNonQuery()
            Catch edb As System.Data.SqlClient.SqlException
                Trace.WriteLine(edb.Message)
                If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                    Dim eSQL As SqlError
                    For Each eSQL In edb.Errors
                        Trace.WriteLine(eSQL.Message, eSQL.Procedure.ToString)
                    Next
                End If
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Dim pIn As SqlParameter
                    For Each pIn In .Parameters
                        Trace.WriteLine(pIn.ParameterName & "=" & pIn.Value.ToString & "(" & pIn.Size & ")")
                    Next
                End If
            End Try
        End With
        _DBConnDelete.Close()
    End Sub
#End Region

#Region "AsynchJobSet"
    Protected Sub AsynchJobSet(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                  ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                  ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                  ByVal InError As Boolean, ByVal Offline As Boolean, _
                                  ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                  ByVal QueuedTime As Integer, _
                                  ByVal JobSize As Integer, ByVal JobId As Integer)

        Dim _DBConnSet As New SqlConnection(_ConnectionString)

        If _DBConnSet.State = ConnectionState.Closed Then
            _DBConnSet.Open()
        End If

        Dim cmdPrintJobSet As New SqlCommand(JobSetStoredProcName, _DBConnSet)
        With cmdPrintJobSet
            .CommandType = CommandType.StoredProcedure
            .Parameters.Add("@PrinterName", SqlDbType.VarChar, FIELDSIZE_PRINTERNAME).Value = PrinterName
            .Parameters.Add("@Username", SqlDbType.VarChar, FIELDSIZE_USERNAME).Value = Username
            .Parameters.Add("@Document", SqlDbType.VarChar, FIELDSIZE_DOCUMENTNAME).Value = Document
            .Parameters.Add("@Machinename", SqlDbType.VarChar, FIELDSIZE_MACHINENAME).Value = Machinename
            .Parameters.AddWithValue("@Pages", Pages)
            .Parameters.AddWithValue("@Copies", Copies)
            .Parameters.AddWithValue("@Submitted", Submitted)
            .Parameters.AddWithValue("@Status", Status)
            .Parameters.AddWithValue("@PagesPrinted", PagesPrinted)
            .Parameters.AddWithValue("@Paused", Paused)
            .Parameters.AddWithValue("@Deleted", Deleted)
            .Parameters.AddWithValue("@Deleting", Deleting)
            .Parameters.AddWithValue("@Printed", Printed)
            .Parameters.AddWithValue("@Printing", Printing)
            .Parameters.AddWithValue("@InError", InError)
            .Parameters.AddWithValue("@Offline", Offline)
            .Parameters.AddWithValue("@PaperOut", PaperOut)
            .Parameters.AddWithValue("@UserInterventionRequired", UserInterventionRequired)
            .Parameters.AddWithValue("@QueuedTime", QueuedTime)
            .Parameters.AddWithValue("@JobSize", JobSize)
            .Parameters.AddWithValue("@JobId", JobId)
            Try
                Dim RowsAffected As Integer
                RowsAffected = .ExecuteNonQuery()
            Catch edb As System.Data.SqlClient.SqlException
                Trace.WriteLine(edb.Message)
                If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                    Dim eSQL As SqlError
                    For Each eSQL In edb.Errors
                        Trace.WriteLine(eSQL.Message, eSQL.Procedure.ToString)
                    Next
                End If
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Dim pIn As SqlParameter
                    For Each pIn In .Parameters
                        Trace.WriteLine(pIn.ParameterName & "=" & pIn.Value.ToString & "(" & pIn.Size & ")")
                    Next
                End If
            End Try
        End With
        _DBConnSet.Close()
    End Sub
#End Region

#Region "AsynchJobWritten"
    Protected Sub AsynchJobWritten(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                     ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                     ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                     ByVal InError As Boolean, ByVal Offline As Boolean, _
                                     ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                     ByVal QueuedTime As Integer, _
                                     ByVal JobSize As Integer, ByVal JobId As Integer)

        Dim _DBConnWrite As New SqlConnection(_ConnectionString)

        If _DBConnWrite.State = ConnectionState.Closed Then
            _DBConnWrite.Open()
        End If

        Dim cmdPrintJobWritten As New SqlCommand(JobWrittenStoredProcName, _DBConnWrite)
        With cmdPrintJobWritten
            .CommandType = CommandType.StoredProcedure
            .Parameters.Add("@PrinterName", SqlDbType.VarChar, FIELDSIZE_PRINTERNAME).Value = PrinterName
            .Parameters.AddWithValue("@JobId", JobId)
            .Parameters.Add("@Username", SqlDbType.VarChar, FIELDSIZE_USERNAME).Value = Username
            .Parameters.Add("@Document", SqlDbType.VarChar, FIELDSIZE_DOCUMENTNAME).Value = Document
            .Parameters.Add("@Machinename", SqlDbType.VarChar, FIELDSIZE_MACHINENAME).Value = Machinename
            .Parameters.AddWithValue("@Pages", Pages)
            .Parameters.AddWithValue("@Copies", Copies)
            .Parameters.AddWithValue("@Submitted", Submitted)
            .Parameters.AddWithValue("@Status", Status)
            .Parameters.AddWithValue("@PagesPrinted", PagesPrinted)
            .Parameters.AddWithValue("@Paused", Paused)
            .Parameters.AddWithValue("@Deleted", Deleted)
            .Parameters.AddWithValue("@Deleting", Deleting)
            .Parameters.AddWithValue("@Printed", Printed)
            .Parameters.AddWithValue("@Printing", Printing)
            .Parameters.AddWithValue("@InError", InError)
            .Parameters.AddWithValue("@Offline", Offline)
            .Parameters.AddWithValue("@PaperOut", PaperOut)
            .Parameters.AddWithValue("@UserInterventionRequired", UserInterventionRequired)
            .Parameters.AddWithValue("@QueuedTime", QueuedTime)
            .Parameters.AddWithValue("@JobSize", JobSize)
            Try
                Dim RowsAffected As Integer
                RowsAffected = .ExecuteNonQuery()
            Catch edb As System.Data.SqlClient.SqlException
                Trace.WriteLine(edb.Message)
                If PrinterMonitoringService.ApplicationTracing.TraceWarning Then
                    Dim eSQL As SqlError
                    For Each eSQL In edb.Errors
                        Trace.WriteLine(eSQL.Message, eSQL.Procedure.ToString)
                    Next
                End If
                If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                    Dim pIn As SqlParameter
                    For Each pIn In .Parameters
                        Trace.WriteLine(pIn.ParameterName & "=" & pIn.Value.ToString & "(" & pIn.Size & ")")
                    Next
                End If
            End Try
        End With
        _DBConnWrite.Close()
    End Sub
#End Region

#End Region

#Region "Asynchronous delegate completion handlers"
    Protected Sub AsynchJobAddComplete(ByVal ar As IAsyncResult)
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobAdd completed at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If
    End Sub

    Protected Sub AsynchJobDeleteComplete(ByVal ar As IAsyncResult)
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobDelete completed at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If
    End Sub

    Protected Sub AsynchJobWrittenComplete(ByVal ar As IAsyncResult)
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobWritten completed at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If
    End Sub

    Protected Sub AsynchJobSetComplete(ByVal ar As IAsyncResult)
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobSet completed at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If
    End Sub
#End Region


End Class