Imports PrinterMonitorListnerbase

'\\ --[PrinterMonitorLogfileListener]--------------------
'\\ Monitors printer change events to a logfile
'\\ (c) 2004 Merrion Computing Ltd
'\\ -----------------------------------------------------
Public Class PrinterMonitorLogfileListener
    Inherits PrinterMonitorListenerBase

#Region "Private members"
    Private _Logfilename As String
#End Region

#Region "Command line startup parameters"
    Public Overrides WriteOnly Property CommandLine() As String
        Set(ByVal Value As String)
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("CommandLine(" & Value & ")", Me.GetType.ToString)
            End If
            _Logfilename = Value
        End Set
    End Property
#End Region

#Region "OnPrinterInformationChanged"
    Public Overrides Sub OnPrinterInformationChanged(ByVal PrinterName As String, ByVal Location As String, ByVal Comment As String, ByVal JobCount As Integer, ByVal IsReady As Boolean, ByVal IsDoorOpen As Boolean, _
        ByVal IsInError As Boolean, ByVal IsInitialising As Boolean, _
        ByVal IsAwaitingManualFeed As Boolean, ByVal IsOutOfToner As Boolean, _
        ByVal IsUnavailable As Boolean, ByVal IsOffline As Boolean, _
        ByVal IsOutOfMemory As Boolean, ByVal IsOutputBinFull As Boolean, _
        ByVal IsPaperJammed As Boolean, ByVal IsOutOfPaper As Boolean, _
        ByVal IsPaused As Boolean, ByVal IsDeletingJob As Boolean, _
        ByVal IsInPowerSave As Boolean, ByVal IsPrinting As Boolean, _
        ByVal IsWaitingOnUserIntervention As Boolean, ByVal IsWarmingUp As Boolean, _
        ByVal ServerName As String, _
        ByVal ShareName As String, _
        ByVal PortName As String, _
        ByVal DriverName As String, _
        ByVal SeperatorFilename As String, _
        ByVal PrintProcessor As String, _
        ByVal DefaultDataType As String, _
        ByVal Parameters As String)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnPrinterInformationChanged", Me.GetType.ToString)
        End If

        If Not _Logfilename = "" Then
            Dim jf As New System.IO.StreamWriter(_Logfilename, True)
            jf.WriteLine("Printer settings changed " & System.DateTime.Now.ToString)
            jf.WriteLine("Printer : " & PrinterName)
            jf.WriteLine("Location : " & Location)
            jf.WriteLine("Comment : " & Comment)
            jf.WriteLine("Pending jobs : " & JobCount.ToString)
            jf.Close()
        End If

    End Sub
#End Region

    Public WriteOnly Property LogfileName() As String
        Set(ByVal Value As String)
            _Logfilename = Value
        End Set
    End Property

    Public Sub New()
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("New ", Me.GetType.ToString)
        End If
    End Sub

#Region "Service events"
    Public Overrides Sub OnStopService()

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnStopService", Me.GetType.ToString)
        End If

    End Sub
#End Region

End Class

'\\ --[PrintJobMonitorLogfileListener]-------------------
'\\ Monitors print job events to a named logfile
'\\ (c) 2004 Merrion Computing Ltd
'\\ -----------------------------------------------------
Public Class PrintJobMonitorLogfileListener
    Inherits PrintJobMonitorListenerBase

#Region "Private members"
    Private _Logfilename As String
#End Region

#Region "Command line startup parameters"
    Public Overrides WriteOnly Property CommandLine() As String
        Set(ByVal Value As String)
            If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
                Trace.WriteLine("CommandLine(" & Value & ")", Me.GetType.ToString)
            End If
            _Logfilename = Value
        End Set
    End Property
#End Region

#Region "OnJobAdded"
    Public Overrides Sub OnJobAdded(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal PaperType As String, _
                                       ByVal Landscape As Boolean, ByVal Colour As Boolean, _
                                       ByVal PaperSource As String, ByVal PrinterResolution As String, _
                                       ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                       ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                       ByVal InError As Boolean, ByVal Offline As Boolean, _
                                       ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                       ByVal DataType As String, _
                                       ByVal PaperWidth As Integer, _
                                       ByVal PaperHeight As Integer, _
                                       ByVal PrinterResolutionX As Integer, _
                                       ByVal PrinterResolutionY As Integer, _
                                       ByVal NotifyUserName As String, _
                                       ByVal PrintProcessorName As String, _
                                       ByVal DriverName As String, _
                                       ByVal Parameters As String, _
                                       ByVal QueuedTime As Integer, _
                                       ByVal JobSize As Integer, _
                                       ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobAdded", Me.GetType.ToString)
        End If

        If Not _Logfilename = "" Then
            Dim jf As New System.IO.StreamWriter(_Logfilename, True)
            jf.WriteLine("Print job added " & System.DateTime.Now.ToString)
            jf.WriteLine("Printer : " & PrinterName)
            jf.WriteLine("Username : " & Username)
            jf.WriteLine("Machine : " & Machinename)
            jf.WriteLine("Document : " & Document)
            jf.WriteLine("Pages : " & Pages.ToString)
            jf.WriteLine("Copies : " & Copies.ToString)
            jf.WriteLine("Submitted: " & Submitted.ToString)
            jf.WriteLine("Paper Type : " & PaperType)
            jf.WriteLine("Job Id : " & JobId.ToString)
            If Landscape Then
                jf.WriteLine("Orientation : Landscape")
            Else
                jf.WriteLine("Orientation : Portrait")
            End If
            If Colour Then
                jf.WriteLine("Colour? : Yes")
            Else
                jf.WriteLine("Colour? : No")
            End If
            jf.WriteLine("Paper Source : " & PaperSource)
            jf.WriteLine("Resolution : " & PrinterResolution)
            If Paused Then jf.WriteLine(" Paused")
            If Printed Then jf.WriteLine(" Printed")
            If Printing Then jf.WriteLine(" Printing")
            If InError Then jf.WriteLine(" In Error")
            If Offline Then jf.WriteLine(" Off line")
            If PaperOut Then jf.WriteLine(" Out of paper")
            If UserInterventionRequired Then jf.WriteLine(" User intervention required")
            jf.Close()
        End If
    End Sub
#End Region

#Region "OnJobDeleted"
    Public Overrides Sub OnJobDeleted(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                         ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                         ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                         ByVal InError As Boolean, ByVal Offline As Boolean, _
                                         ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                         ByVal QueuedTime As Integer, ByVal JobSize As Integer, ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobDeleted", Me.GetType.ToString)
        End If

        If Not _Logfilename = "" Then
            Dim jf As New System.IO.StreamWriter(_Logfilename, True)
            jf.WriteLine("Print job deleted " & System.DateTime.Now.ToString)
            jf.WriteLine("Printer : " & PrinterName)
            jf.WriteLine("Username : " & Username)
            jf.WriteLine("Machine : " & Machinename)
            jf.WriteLine("Document : " & Document)
            jf.WriteLine("Pages : " & Pages.ToString)
            jf.WriteLine("Copies : " & Copies.ToString)
            jf.WriteLine("Submitted: " & Submitted.ToString)
            jf.WriteLine("Status: " & Status)
            jf.WriteLine("Pages printed: " & PagesPrinted.ToString)
            If Paused Then jf.WriteLine(" Paused")
            If Printed Then jf.WriteLine(" Printed")
            If Printing Then jf.WriteLine(" Printing")
            If InError Then jf.WriteLine(" In Error")
            If Offline Then jf.WriteLine(" Off line")
            If PaperOut Then jf.WriteLine(" Out of paper")
            If UserInterventionRequired Then jf.WriteLine(" User intervention required")
            jf.Close()
        End If
    End Sub
#End Region

#Region "OnJobSet"
    Public Overrides Sub OnJobSet(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                   ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                   ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                   ByVal InError As Boolean, ByVal Offline As Boolean, _
                                   ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                   ByVal QueuedTime As Integer, ByVal JobSize As Integer, ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobSet", Me.GetType.ToString)
        End If

        If Not _Logfilename = "" Then
            Dim jf As New System.IO.StreamWriter(_Logfilename, True)
            jf.WriteLine("Print job set " & System.DateTime.Now.ToString)
            jf.WriteLine("Printer : " & PrinterName)
            jf.WriteLine("Username : " & Username)
            jf.WriteLine("Machine : " & Machinename)
            jf.WriteLine("Document : " & Document)
            jf.WriteLine("Pages : " & Pages.ToString)
            jf.WriteLine("Copies : " & Copies.ToString)
            jf.WriteLine("Submitted: " & Submitted.ToString)
            jf.WriteLine("Status: " & Status)
            jf.WriteLine("Pages printed: " & PagesPrinted.ToString)
            If Paused Then jf.WriteLine(" Paused")
            If Printed Then jf.WriteLine(" Printed")
            If Printing Then jf.WriteLine(" Printing")
            If InError Then jf.WriteLine(" In Error")
            If Offline Then jf.WriteLine(" Off line")
            If PaperOut Then jf.WriteLine(" Out of paper")
            If UserInterventionRequired Then jf.WriteLine(" User intervention required")
            jf.Close()
        End If
    End Sub
#End Region

#Region "OnJobWritten"
    Public Overrides Sub OnJobWritten(ByVal PrinterName As String, ByVal Username As String, ByVal Document As String, ByVal Machinename As String, ByVal Pages As Integer, ByVal Copies As Integer, ByVal Submitted As DateTime, ByVal Status As String, ByVal PagesPrinted As Integer, _
                                     ByVal Paused As Boolean, ByVal Deleted As Boolean, ByVal Deleting As Boolean, _
                                     ByVal Printed As Boolean, ByVal Printing As Boolean, _
                                     ByVal InError As Boolean, ByVal Offline As Boolean, _
                                     ByVal PaperOut As Boolean, ByVal UserInterventionRequired As Boolean, _
                                     ByVal QueuedTime As Integer, ByVal JobSize As Integer,ByVal JobId As Integer)

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobWritten", Me.GetType.ToString)
        End If

        If Not _Logfilename = "" Then
            Dim jf As New System.IO.StreamWriter(_Logfilename, True)
            jf.WriteLine("Print job written " & System.DateTime.Now.ToString)
            jf.WriteLine("Printer : " & PrinterName)
            jf.WriteLine("Username : " & Username)
            jf.WriteLine("Machine : " & Machinename)
            jf.WriteLine("Document : " & Document)
            jf.WriteLine("Pages : " & Pages.ToString)
            jf.WriteLine("Copies : " & Copies.ToString)
            jf.WriteLine("Submitted: " & Submitted.ToString)
            jf.WriteLine("Status: " & Status)
            jf.WriteLine("Pages printed: " & PagesPrinted.ToString)
            If Paused Then jf.WriteLine(" Paused")
            If Printed Then jf.WriteLine(" Printed")
            If Printing Then jf.WriteLine(" Printing")
            If InError Then jf.WriteLine(" In Error")
            If Offline Then jf.WriteLine(" Off line")
            If PaperOut Then jf.WriteLine(" Out of paper")
            If UserInterventionRequired Then jf.WriteLine(" User intervention required")
            jf.Close()
        End If
    End Sub
#End Region

    Public WriteOnly Property LogfileName() As String
        Set(ByVal Value As String)
            _Logfilename = Value
        End Set
    End Property

    Public Sub New()
        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("New ", Me.GetType.ToString)
        End If
    End Sub

#Region "Service events"
    Public Overrides Sub OnStopService()

        If PrinterMonitoringService.ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnStopService", Me.GetType.ToString)
        End If

    End Sub
#End Region
End Class