Imports System.ServiceProcess
Imports System.Drawing
Imports System.Data.SqlClient
Imports PrinterMonitorService.CustomConfigurationSectionReaders


<System.ServiceProcess.ServiceProcessDescriptionAttribute("Printer Usage Monitoring")> _
Public Class PrinterMonitoringService
    Inherits System.ServiceProcess.ServiceBase

#Region "Private properties"
    Dim Settings As New PrinterMonitorServiceSettings

    Dim PrinterEventListeners As New NamedListenerDictionary
    Dim PrintJobEventListeners As New NamedListenerDictionary

#End Region

#Region "Asynchronous execution delegates"
    Public Delegate Sub OnStartDelegate()
    Public Delegate Sub OnStopDelegate()

    Public Delegate Sub OnJobEventDelegate(ByVal EventType As PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes, ByVal job As PrinterQueueWatch.PrintJob)
    Public Delegate Sub OnPrinterEventDelegate(ByVal Printer As PrinterQueueWatch.PrinterInformation)

#End Region

#Region " Component Designer generated code "

    Public Sub New()
        MyBase.New()

        ' This call is required by the Component Designer.
        InitializeComponent()

        ' Add any initialization after the InitializeComponent() call

    End Sub

    'UserService overrides dispose to clean up the component list.
    Protected Overloads Overrides Sub Dispose(ByVal disposing As Boolean)
        If disposing Then
            If Not (components Is Nothing) Then
                components.Dispose()
            End If
        End If
        MyBase.Dispose(disposing)
    End Sub

    ' The main entry point for the process
    <MTAThread()> _
    Shared Sub Main()
        Dim ServicesToRun() As System.ServiceProcess.ServiceBase

        ' More than one NT Service may run within the same process. To add
        ' another service to this process, change the following line to
        ' create a second service object. For example,
        '
        '   ServicesToRun = New System.ServiceProcess.ServiceBase () {New Service1, New MySecondUserService}
        '
        ServicesToRun = New System.ServiceProcess.ServiceBase() {New PrinterMonitoringService}

        System.ServiceProcess.ServiceBase.Run(ServicesToRun)
    End Sub

    'Required by the Component Designer
    Private components As System.ComponentModel.IContainer

    ' NOTE: The following procedure is required by the Component Designer
    ' It can be modified using the Component Designer.  
    ' Do not modify it using the code editor.
    Friend WithEvents pPrinter As PrinterQueueWatch.PrinterMonitorComponent
    Friend WithEvents NotifyIcon_PrintMonitor As System.Windows.Forms.NotifyIcon
    <System.Diagnostics.DebuggerStepThrough()> Private Sub InitializeComponent()
        Me.components = New System.ComponentModel.Container
        Dim resources As System.ComponentModel.ComponentResourceManager = New System.ComponentModel.ComponentResourceManager(GetType(PrinterMonitoringService))
        Me.pPrinter = New PrinterQueueWatch.PrinterMonitorComponent(Me.components)
        Me.NotifyIcon_PrintMonitor = New System.Windows.Forms.NotifyIcon(Me.components)
        '
        'pPrinter
        '
        Me.pPrinter.MonitorJobEventInformationLevel = PrinterQueueWatch.PrinterMonitorComponent.MonitorJobEventInformationLevels.MaximumJobInformation
        Me.pPrinter.ThreadTimeout = -1
        '
        'NotifyIcon_PrintMonitor
        '
        Me.NotifyIcon_PrintMonitor.Icon = CType(resources.GetObject("NotifyIcon_PrintMonitor.Icon"), System.Drawing.Icon)
        Me.NotifyIcon_PrintMonitor.Text = "Printer Monitoring"
        Me.NotifyIcon_PrintMonitor.Visible = True
        '
        'PrinterMonitoringService
        '
        Me.CanShutdown = True
        Me.ServiceName = "Printer Monitoring Service"

    End Sub

#End Region

#Region "Application trace switch"
    Public Shared ApplicationTracing As New System.Diagnostics.TraceSwitch("Application", "Printer Watch test application tracing")
#End Region

#Region "ServiceProcess.ServiceBase overrides"

    Protected Overrides Sub OnStart(ByVal args() As String)
        ' Add code here to start your service. This method should set things
        ' in motion so your service can do its work.
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Service OnStart called", Me.GetType.ToString)
        End If

        Dim OnStartDelegateInst As OnStartDelegate = AddressOf OnStartAsynch
        OnStartDelegateInst.BeginInvoke(AddressOf OnStartCallback, OnStartDelegateInst)

    End Sub

    Protected Overrides Sub OnStop()
        ' Add code here to perform any tear-down necessary to stop your service.
        Dim OnStopDelegateInst As OnStopDelegate = AddressOf OnStopAsynch
        OnStopDelegateInst.BeginInvoke(AddressOf OnStopCallback, OnStopDelegateInst)

        pPrinter.Dispose()

    End Sub

#End Region

#Region "Asynchronous start/stop handling"
    Private Sub OnStartAsynch()

        Try
            PrinterEventListeners = CustomConfigurationSectionReaders.PrinterEventListenersSectionReader.GetPrinterEventListenersNamedListenerDictionary
        Catch ExPrinterEventListeners As Exception
            If ApplicationTracing.TraceError Then
                Trace.WriteLine(ExPrinterEventListeners.ToString, Me.GetType.ToString)
            End If
        End Try

        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine(PrinterEventListeners.Count & " printer event listeners added", Me.GetType.ToString)
        End If
        Try
            PrintJobEventListeners = CustomConfigurationSectionReaders.PrintJobEventListenersSectionReader.GetPrintJobNamedListenerDictionary
        Catch ExPrintJobEventListeners As Exception
            If ApplicationTracing.TraceError Then
                Trace.WriteLine(ExPrintJobEventListeners.ToString, Me.GetType.ToString)
            End If
        End Try


        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine(PrintJobEventListeners.Count & " printer event listeners added", Me.GetType.ToString)
        End If

        With Settings
            pPrinter.MonitorJobAddedEvent = .MonitorJobAddedEvent
            pPrinter.MonitorJobDeletedEvent = .MonitorDeletedEvent
            pPrinter.MonitorJobSetEvent = .MonitorJobSetEvent
            pPrinter.MonitorJobWrittenEvent = .MonitorJobWrittenEvent
            pPrinter.MonitorPrinterChangeEvent = .MonitorPrinterChangeEvent
            pPrinter.MonitorJobEventInformationLevel = .MonitorJobEventInformationLevel
            'me.NotifyIcon_PrintMonitor.Visible 
        End With

        Dim _PrinterAdded As Boolean
        Dim MonitoredPrinter As MonitoredPrinter
        If Settings.ReadSettingsFromDB Then
            '\\ Get the printers to be added from the database
            Dim _DBConn As New SqlConnection(Settings.ConnectionString)
            If _DBConn.State <> ConnectionState.Open Then
                _DBConn.Open()
            End If

            Dim sSQL As String
            sSQL = "SELECT DISTINCT Name FROM dbo.Printers WHERE Monitor = 1 AND [Server Name] = '" & _DBConn.WorkstationId & "'"
            Try
                Dim PrinterQuery As SqlCommand = New SqlCommand(sSQL, _DBConn)
                Dim PrinterNameReader As SqlDataReader = PrinterQuery.ExecuteReader
                While PrinterNameReader.Read
                    pPrinter.AddPrinter(PrinterNameReader.GetString(0))
                End While
                _PrinterAdded = True
            Catch ex As Exception
                If ApplicationTracing.TraceError Then
                    Trace.WriteLine("Failed to read printers for this server", Me.GetType.ToString)
                    Trace.WriteLine(ex.ToString, Me.GetType.ToString)
                End If
            Finally
                '\\ Close the database connection, if it worked or not...
                _DBConn.Close()
            End Try
        Else
            For Each MonitoredPrinter In MonitoredPrintersSectionReader.GetMonitoredPrintersCollection
                If ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Adding printer" & MonitoredPrinter.DeviceName, Me.GetType.ToString)
                End If
                Try
                    pPrinter.AddPrinter(MonitoredPrinter.DeviceName)
                    _PrinterAdded = True
                Catch ex As Exception
                    '\\ Note the problem and skip this printer
                    If ApplicationTracing.TraceError Then
                        Trace.WriteLine(ex.Message, Me.GetType.ToString)
                    End If
                End Try
            Next
        End If

        '\\ If no printers where specified, add all known printers
        If Not _PrinterAdded Then
            Dim PrinterName As String
            For Each PrinterName In System.Drawing.Printing.PrinterSettings.InstalledPrinters
                If ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Adding printer" & PrinterName, Me.GetType.ToString)
                End If
                Try
                    pPrinter.AddPrinter(PrinterName)
                Catch exs As Exception
                    If ApplicationTracing.TraceError Then
                        Trace.Write(exs.Message, Me.GetType.ToString)
                    End If
                End Try
            Next
        End If
    End Sub

    Private Sub OnStopAsynch()
        If PrintJobEventListeners.Count > 0 Then
            Dim PrintJobEventListener As NamedListener
            For Each PrintJobEventListener In PrintJobEventListeners.Values
                Call PrintJobEventListener.Instance.OnStopService()
            Next
        End If

        If PrinterEventListeners.Count > 0 Then
            Dim PrinterEventListener As NamedListener
            For Each PrinterEventListener In PrinterEventListeners.Values
                Call PrinterEventListener.Instance.OnStopService()
            Next
        End If

        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Service OnStop called", Me.GetType.ToString)
        End If

    End Sub

    Private Sub OnStartCallback(ByVal ar As IAsyncResult)
        ar.AsyncState.EndInvoke(ar)
    End Sub

    Private Sub OnStopCallback(ByVal ar As IAsyncResult)
        ar.AsyncState.EndInvoke(ar)
    End Sub

#End Region

#Region "Printer event handling"

    Private Sub pPrinter_JobAdded(ByVal sender As Object, ByVal e As PrinterQueueWatch.PrintJobEventArgs) Handles pPrinter.JobAdded
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Printer job added", Me.GetType.ToString)
        End If
        Dim OnJobEventHandler As OnJobEventDelegate = AddressOf OnJobEvent
        OnJobEventHandler.BeginInvoke(PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobAddedEvent, DirectCast(e, PrinterQueueWatch.PrintJobEventArgs).PrintJob, AddressOf OnJobEventCallback, OnJobEventHandler)
    End Sub

    Private Sub pPrinter_JobDeleted(ByVal sender As Object, ByVal e As PrinterQueueWatch.PrintJobEventArgs) Handles pPrinter.JobDeleted
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Printer job deleted", Me.GetType.ToString)
        End If
        Dim OnJobEventHandler As OnJobEventDelegate = AddressOf OnJobEvent
        OnJobEventHandler.BeginInvoke(PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobDeletedEvent, DirectCast(e, PrinterQueueWatch.PrintJobEventArgs).PrintJob, AddressOf OnJobEventCallback, OnJobEventHandler)
    End Sub

    Private Sub pPrinter_JobSet(ByVal sender As Object, ByVal e As PrinterQueueWatch.PrintJobEventArgs) Handles pPrinter.JobSet
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Printer job set", Me.GetType.ToString)
        End If
        Dim OnJobEventHandler As OnJobEventDelegate = AddressOf OnJobEvent
        OnJobEventHandler.BeginInvoke(PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobSetEvent, DirectCast(e, PrinterQueueWatch.PrintJobEventArgs).PrintJob, AddressOf OnJobEventCallback, OnJobEventHandler)

    End Sub

    Private Sub pPrinter_JobWritten(ByVal sender As Object, ByVal e As PrinterQueueWatch.PrintJobEventArgs) Handles pPrinter.JobWritten
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Printer job written", Me.GetType.ToString)
        End If
        Dim OnJobEventHandler As OnJobEventDelegate = AddressOf OnJobEvent
        OnJobEventHandler.BeginInvoke(PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobWrittenEvent, DirectCast(e, PrinterQueueWatch.PrintJobEventArgs).PrintJob, AddressOf OnJobEventCallback, OnJobEventHandler)
    End Sub

    Private Sub pPrinter_PrinterInformationChanged(ByVal sender As Object, ByVal e As PrinterQueueWatch.PrinterEventArgs) Handles pPrinter.PrinterInformationChanged
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("Printer information changed", Me.GetType.ToString)
        End If
        Dim OnPrinterEventHandler As OnPrinterEventDelegate = AddressOf OnPrinterEvent
        OnPrinterEventHandler.BeginInvoke(DirectCast(e, PrinterQueueWatch.PrinterEventArgs).PrinterInformation, AddressOf OnPrinterEventCallback, OnPrinterEventHandler)
    End Sub

#End Region

#Region "Private methods"
    Private Sub OnJobEvent(ByVal EventType As PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes, ByVal job As PrinterQueueWatch.PrintJob)
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobEvent (" & EventType.ToString & ") at " & Date.Now.TimeOfDay.ToString, Me.GetType.ToString)
        End If
        '\\ Iterate through all the printjob listeners, calling their OnJobAdded...OnJobWritten event as appropriate
        If PrintJobEventListeners.Count > 0 Then
            Dim PrintJobEventListener As NamedListener
            For Each PrintJobEventListener In PrintJobEventListeners.Values
                If ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Job event handled : " & EventType.ToString, PrintJobEventListener.GetType.ToString)
                End If
                With job
                    Select Case EventType
                        Case PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobAddedEvent
                            Try
                                PrintJobEventListener.Instance.OnJobAdded(.PrinterName, .UserName, .Document, .MachineName, .TotalPages, .Copies, .Submitted, .PaperKind.ToString, _
                                        .Landscape, .Color, .PaperSource.ToString, .PrinterResolutionKind.ToString, _
                                        .Paused, .Deleted, .Deleting, _
                                        .Printed, .Printing, .InError, .Offline, _
                                        .PaperOut, .UserInterventionRequired, _
                                        .DataType, .PaperWidth, .PaperLength, _
                                        .PrinterResolutionX, .PrinterResolutionY, _
                                        .NotifyUserName, .PrintProcessorName, _
                                        .DriverName, .Parameters, _
                                        .QueuedTime, .JobSize, .JobId)
                            Catch e As Exception
                                If ApplicationTracing.TraceError Then
                                    Trace.WriteLine("OnJobAdded : " & e.ToString, PrintJobEventListener.GetType.ToString)
                                End If
                            End Try
                        Case PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobDeletedEvent
                            Try
                                PrintJobEventListener.Instance.OnJobDeleted(.PrinterName, .UserName, .Document, .MachineName, .TotalPages, .Copies, _
                                      .Submitted, .StatusDescription, .PagesPrinted, _
                                      .Paused, .Deleted, .Deleting, _
                                      .Printed, .Printing, .InError, .Offline, _
                                      .PaperOut, .UserInterventionRequired, _
                                      .QueuedTime, .JobSize, .JobId)
                            Catch e As Exception
                                If ApplicationTracing.TraceError Then
                                    Trace.WriteLine("OnJobDelete : " & e.ToString, PrintJobEventListener.GetType.ToString)
                                End If
                            End Try
                        Case PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobSetEvent
                            Try
                                PrintJobEventListener.Instance.OnJobSet(.PrinterName, .UserName, .Document, .MachineName, .TotalPages, .Copies, _
                                      .Submitted, .StatusDescription, .PagesPrinted, _
                                      .Paused, .Deleted, .Deleting, _
                                      .Printed, .Printing, .InError, .Offline, _
                                      .PaperOut, .UserInterventionRequired, _
                                       .QueuedTime, .JobSize, .JobId)
                            Catch e As Exception
                                If ApplicationTracing.TraceError Then
                                    Trace.WriteLine("OnJobSet : " & e.ToString, PrintJobEventListener.GetType.ToString)
                                End If
                            End Try
                        Case PrinterQueueWatch.PrintJobEventArgs.PrintJobEventTypes.JobWrittenEvent
                            Try
                                PrintJobEventListener.Instance.OnJobWritten(.PrinterName, .UserName, .Document, .MachineName, .TotalPages, .Copies, _
                                      .Submitted, .StatusDescription, .PagesPrinted, _
                                      .Paused, .Deleted, .Deleting, _
                                      .Printed, .Printing, .InError, .Offline, _
                                      .PaperOut, .UserInterventionRequired, _
                                      .QueuedTime, .JobSize, .JobId)
                            Catch e As Exception
                                If ApplicationTracing.TraceError Then
                                    Trace.WriteLine("OnJobWritten : " & e.ToString, PrintJobEventListener.GetType.ToString)
                                End If
                            End Try
                    End Select
                End With
            Next
        Else
            If ApplicationTracing.TraceWarning Then
                Trace.WriteLine("No print job event listeners attached", Me.GetType.ToString)
            End If
        End If
    End Sub

    Private Sub OnPrinterEvent(ByVal Printer As PrinterQueueWatch.PrinterInformation)
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnPrinterEvent ", Me.GetType.ToString)
        End If
        If PrinterEventListeners.Count > 0 Then
            If ApplicationTracing.TraceWarning Then
                Trace.WriteLine(PrinterEventListeners.Count & " printer event listeners attached", Me.GetType.ToString)
            End If
            Dim PrinterEventListener As NamedListener
            For Each PrinterEventListener In PrinterEventListeners.Values
                If ApplicationTracing.TraceVerbose Then
                    Trace.WriteLine("Printer change event handled ", PrinterEventListener.GetType.ToString)
                End If
                With Printer
                    Try
                        PrinterEventListener.Instance.OnPrinterInformationChanged(.PrinterName, .Location, .Comment, .JobCount, _
                                  .IsReady, .IsDoorOpen, _
                                  .IsInError, .IsInitialising, _
                                  .IsAwaitingManualFeed, .IsOutOfToner, _
                                  .IsUnavailable, .IsOffline, _
                                  .IsOutOfmemory, .IsOutputBinFull, _
                                  .IsPaperJammed, .IsOutOfPaper, _
                                  .Paused, .IsDeletingJob, _
                                  .IsInPowerSave, .IsPrinting, _
                                  .IsWaitingOnUserIntervention, .IsWarmingUp, _
                                  .ServerName, .ShareName, _
                                  .PortName, .DriverName, _
                                  .SeperatorFilename, _
                                  .PrintProcessor, _
                                  .DefaultDataType, _
                                  .Parameters)
                    Catch e As Exception
                        If ApplicationTracing.TraceError Then
                            Trace.WriteLine("OnPrinterInformationChanged : " & e.ToString, PrinterEventListener.GetType.ToString)
                        End If
                    End Try
                End With
            Next
        Else
            If ApplicationTracing.TraceWarning Then
                Trace.WriteLine("No printer event listeners attached", Me.GetType.ToString)
            End If
        End If
    End Sub

    Private Sub OnJobSpoolfileParsedEvent(ByVal ParsedFile As SpoolMonitoringIPC.SpoolParseResponse)
        If ApplicationTracing.TraceVerbose Then
            Trace.WriteLine("OnJobSpoolfileParsedEvent ", Me.GetType.ToString)
            Trace.Indent()
            Trace.WriteLine("Printer: " & ParsedFile.PrinterName, Me.GetType.ToString)
            Trace.WriteLine("Job Id: " & ParsedFile.JobId.ToString, Me.GetType.ToString)
            Trace.WriteLine("True page count: " & ParsedFile.TruePageCount, Me.GetType.ToString)
            Trace.Unindent()
        End If

    End Sub

    Private Sub OnJobEventCallback(ByVal ar As IAsyncResult)
        ar.AsyncState.EndInvoke(ar)
    End Sub

    Private Sub OnPrinterEventCallback(ByVal ar As IAsyncResult)
        ar.AsyncState.EndInvoke(ar)
    End Sub

#End Region

End Class
